/****************************************************************************
 MYTH - Channel System
 Coded by Saruk (04/09/99)
 ---
 MYTH's channel system with as much functionality as we can muster. Almost
 ability to do IRC-ish things with the channel.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

CHANNEL *channel_list = NULL;

void __delete_channel(channel)
	CHANNEL *channel;
{
	CHANNEL *c, *cprev = NULL;
	
	for(c = channel_list; c; c = c->next) {
		if(c == channel)
			break;
		cprev = c;
	}
	
	if(!c)
		return;
	
	if(!cprev)
		channel_list = c->next;
	else
		cprev->next = c->next;
	
	if(c->name)
		free(c->name);
	if(c->desc)
		free(c->desc);
	if(c->title)
		free(c->title);
	if(c->locks)
		free(c->locks);
	if(c->alias)
		free(c->alias);
	if(c->colour)
		free(c->colour);

	free(c);		
}

void __remove_channel(player, channel)
	object *player;
	CHANNEL *channel;
{
	CHANNEL *c, *cprev = NULL;
	
	for(c = player->channels; c; c=c->next) {
		if(c == channel)
			break;
		cprev = c;
	}
	
	if(!c)
		return;
	
	if(!cprev)
		player->channels = c->next;
	else
		cprev->next = c->next;
	
	if(c->name)
		free(c->name);
	if(c->desc)
		free(c->desc);
	if(c->title)
		free(c->title);
	if(c->locks)
		free(c->locks);
	if(c->alias)
		free(c->alias);
	if(c->colour)
		free(c->colour);
	
	free(c);
}

/** Clear all channels from an object **/
void clear_channels(player)
	object *player;
{
	CHANNEL *c, *cnext;
	
	for(c = player->channels; c; c = cnext) {
		cnext = c->next;
		notify_channel(NULL, c->chobj, pstr("|R+|* |X|%s |C|has left this channel",
			color_name(player, -1)));
		__remove_channel(player, c);
	}
}
	
/** Return first free channel reference id **/
int channel_ref()
{
	CHANNEL *c;
	int ctr = 0;
	
	for(c = channel_list; c; c=c->next)
		ctr++;
	
	return ctr;
}

/** Create a whole new channel **/
CHANNEL *__create_channel(name, desc)
	char *name;
	char *desc;
{
	CHANNEL *newc;
	
	GENERATE(newc, CHANNEL);
	newc->ref = channel_ref();
	newc->oref = 0;
	newc->owner = NULL;
	newc->flags = 0;
	newc->powlvl = 0;
	newc->status = CHANNEL_CONSTRUCTION;
	newc->chdef = FALSE;
	
	newc->name = NULL;
	newc->desc = NULL;
	newc->title = NULL;
	newc->locks = NULL;
	newc->alias = NULL;
	newc->colour = NULL;
	SET(&newc->name, name);
	SET(&newc->desc, desc);
	SET(&newc->colour, "X");
	
	newc->next = channel_list;
	channel_list = newc;
	newc->chobj = newc;
	newc->player = 0;
	
	return(newc);	
}

CHANNEL *__add_channel(player, channel)
	object *player;
	CHANNEL *channel;
{
	CHANNEL *newc;
	
	GENERATE(newc, CHANNEL);
	newc->ref = channel->ref;
	newc->name = NULL;
	newc->desc = NULL;
	newc->title = NULL;
	newc->locks = NULL;
	newc->alias = NULL;
	newc->colour = NULL;
	SET(&newc->name, channel->name);
	SET(&newc->alias, channel->alias);
	SET(&newc->colour, channel->colour);
	
	newc->owner = channel->owner;
	newc->oref = channel->oref;
	newc->flags = channel->flags;
	newc->powlvl = channel->powlvl;
	newc->status = channel->status;
	newc->chdef = FALSE;
	newc->chobj = channel;
	newc->player = player;
	
	newc->next = player->channels;
	player->channels = newc;
	
	return(newc);
}

int can_setchannel(channel)
	CHANNEL *channel;
{
	if(!channel->name || !channel->desc || !channel->colour || !channel->alias ||
		!channel->title)
		return FALSE;
	
	return TRUE;
}



/** This will see if a channel actually exists on the channel list. **/
/* Returns (NULL) if not */
CHANNEL *__find_channel(matchname)
	char *matchname;
{
	CHANNEL *c;
	int ref;
	
	ref = match_num(matchname);
	
	for(c = channel_list; c; c=c->next)
		if(!strcmpm(c->name, matchname) || ref == c->ref)
			return(c);
	
	return(NULL);
}

/** Does a player have this channel? **/
CHANNEL *__has_channel(player, channel)
	OBJECT *player;
	CHANNEL *channel;
{
	CHANNEL *c;
	
	for(c = player->channels; c; c=c->next)
		if(c->chobj == channel)
			return(c);
	
	return(NULL);
}

/** Completely purge a channel's occupants **/
void __purge_channel(channel)
	CHANNEL *channel;
{
	OBJECT *o;
	CHANNEL *c;
	
	for(o = player_list; o; o=o->next)
		if((c = __has_channel(o, channel)))
			__remove_channel(o, c);
}

/* This will check for locks using pchannel... */
void notify_channel(pchannel, channel, message)
	CHANNEL *pchannel;
	CHANNEL *channel;
	char *message;
{
	CHANNEL *c;
	OBJECT *o;
	char buf[MAX_BUFSIZE];
	
	for(o = player_list; o; o = o->next)
		for(c = o->channels; c; c = c->next) 
			if(c->chobj == channel && c->status == CHANNEL_ON) {
				if( (pchannel && (!parse_lock(o, pchannel->locks) || !parse_lock(pchannel->player, c->locks))) )
					continue;
				sprintf(buf, "|W+|[|%s|%s|W+|]|X| %s", c->colour, channel->name, message);
				notify(o, buf);
			}
}
/** Does player have control over the channel? **/
int channel_control(player, channel)
	object *player;
	CHANNEL *channel;
{
	if(player == channel->owner)
		return 1;
	
	if(can_control(player, channel->owner))
		return 1;
// Make room to allow for channel owners to extend priveleges to other users.	

	return 0;
}

int name_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	CHANNEL *check, *c;
	object *o;
	
	if(strchr(value, ' '))
		return M_EILLNAME;	
		
	if( (check = __find_channel(value)) && (check != channel) )
		return M_ECHSHDW;
	
	SET(&channel->name, value);
	for(o = player_list; o; o=o->next)
		for(c = o->channels; c; c=c->next)
			if(c->chobj == channel)
				SET(&c->name, channel->name);
			
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|The channel is now '|%s|%s|C|'.",
		channel->colour, channel->name));
	
	return TRUE;
}

int desc_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	SET(&channel->desc, value);
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s description is now '|X|%s|C|'.",
		channel->colour, channel->name, channel->desc));
		
	return TRUE;
}

int title_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	if(!strcmp(value, "NULL"))
		channel->title = NULL;
	else
		SET(&channel->title, value);
	
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s title is now '|X|%s|C|'.",
		channel->colour, channel->name, (channel->title) ? channel->title : "|R+|NONE"));
	
	return TRUE;
}

int lock_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	if(!strcmpm(value, "NULL"))
		channel->locks = NULL;
	else
		SET(&channel->locks, value);
		
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s locks set to|W+|:|X| %s",
		channel->colour, channel->name, (channel->locks) ? channel->locks : "|R+|NONE"));
	
	return TRUE;
}

int alias_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	CHANNEL *check;
	
	for(check = channel_list; check; check = check->next)
		if(!strcmpm(check->alias, value) && check != channel)
			return M_ECHALSHDW;
	
	SET(&channel->alias, value);
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s alias set to|W+|: |X|%s",
		channel->colour, channel->name, channel->alias));
	
	return TRUE;
}

int player_alias_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	CHANNEL *c;
	
	for(c = player->channels; c; c=c->next)
		if(!strcmpm(c->alias, value) && c != channel)
			return M_ECHALSHDW;
	
	SET(&channel->alias, value);
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s alias set to|W+|: |X|%s",
		channel->colour, channel->name, channel->alias));
	
	return TRUE;
}

int colour_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	SET(&channel->colour, value);
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|Channel is now coloured|W+|: |%s|%s",
		channel->colour, channel->name));
	
	return TRUE;
}

int flag_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	char buf[MAX_BUFSIZE];
	int ctr, tag = 0, check;
	
	if(*value == '!') {
		strcpy(buf, value + 1);	
		tag = 1;
	}
	
	if(strlen(buf) == 1)
		check = buf[0];
		
	for(ctr = 0; chflag_table[ctr].name; ctr++)
		if( (check == chflag_table[ctr].parsed) || string_prefix(buf, chflag_table[ctr].name) ) {
			if(tag)
				channel->flags &= ~chflag_table[ctr].flag;
			else
				channel->flags |= chflag_table[ctr].flag;
			notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s channel flag |Y+|%s |C|%s.",
				channel->colour, channel->name, chflag_table[ctr].name, (tag) ? "unset":"set"));
			return TRUE;
		}
	
	return M_ENOTFOUND;
}

int powlvl_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	int powerlevel;

	if (!isanumber(value)) 
		return M_ENOTANUM;
	
	powerlevel = match_num(value);
	
	if(powerlevel < MIN_POWER)
		return M_EINSUFF;
	else if (powerlevel > player->powlvl || powerlevel > MAX_POWER)
		return M_EEXCESS;

	channel->powlvl = powerlevel;		
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|Powerlevel for |%s|%s |C| set to|W+|: |Y+|%d",
		channel->colour, channel->name, channel->powlvl));
			
	return TRUE;		
}

int status_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	char buf[MAX_BUFSIZE];

	sprintf(buf, "|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s status set to|W+|:",
		channel->colour, channel->name);
	
	if(string_prefix(value, "construction")) {
		channel->status = CHANNEL_CONSTRUCTION;
		sprintf(buf, "%s |Y+|Under Construction", buf);
	} else if(string_prefix(value, "on")) {
		if(!can_setchannel(channel))
			return M_ECHNOTDONE;

		channel->status = CHANNEL_ON;
		sprintf(buf, "%s |Y+|ON", buf);
	} else if(string_prefix(value, "off")) {
		if(!can_setchannel(channel))
			return M_ECHNOTDONE;
			
		channel->status = CHANNEL_OFF;
		sprintf(buf, "%s |Y+|OFF", buf);
	} else {
		return M_ECHINVSTAT;
	}
	
	notify(player, buf);
	return TRUE;
}

int default_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	char buf[MAX_BUFSIZE];
	
	sprintf(buf, "|C+|[|Y+|CHANNEL|C+|]|W+|: |%s|%s|C|'s default set to |W+|:",
		channel->colour, channel->name);

	if(string_prefix(value, "true")) {
		channel->chdef = TRUE;
		sprintf(buf, "%s |Y+|TRUE", buf);
	} else if (string_prefix(value, "false")) {
		channel->chdef = FALSE;
		sprintf(buf, "%s |Y+|FALSE", buf);
	} else {
		return M_ECHINVDEF;
	}

	notify(player, buf);
	return TRUE;		
}

int owner_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	object *newowner;
	
	if(!(newowner = match_player(value)))
		return M_ENOPLAY;
	
	if(!can_control(player, newowner))
		return M_ENOCNTRL;
	
	channel->owner = newowner;
	channel->oref = newowner->ref;
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |X|%s |C|is now the owner of |%s|%s",
		color_name(newowner, -1), channel->colour, channel->name));
		
	return TRUE;
}

int boot_channel(player, channel, value)
	object *player;
	CHANNEL *channel;
	char *value;
{
	object *o;
	CHANNEL *c;
	
	if(!(o = match_player(value)))
		return M_ENOPLAY;

	if(!(c = __has_channel(o, channel)))
		return M_ECHNOTON;
	
	__remove_channel(o, c);
	notify(o, pstr("|C|You have been |R+|booted|C| from channel |%s|%s",
		channel->colour, channel->name));
	notify(player, pstr("|X|%s |R+|booted|C| from channel.", color_name(o, -1)));	
	notify_channel(NULL, channel, pstr("|R+|* |X|%s |C|has been |R+|booted |C|from this channel.",
		color_name(o, -1)));
	return TRUE;
}

/** Set an actual channel, not the player's mirror of said channel **/
int do_channel_set(player, matchname, checkfor)
	object *player;
	char *matchname;
	char *checkfor;
{
	CHANNEL *c;
	char argument[MAX_BUFSIZE];

	strcpy(argument, checkfor);
	if(!(c = __find_channel(matchname))) 
		return M_ECHNOTFOUND;
	
	if(!*(front(argument)) || !*(restof(argument))) 
		return M_EINVARG;
	
	if(!channel_control(player, c)) 
		return M_ENOCNTRL;

	if(!strcmpm(front(argument), "name")) 
		return name_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "desc"))
		return desc_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "title"))
		return title_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "locks") || !strcmpm(front(argument), "lock"))
		return lock_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "alias"))
		return alias_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "colour") || !strcmpm(front(argument), "color"))
		return colour_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "flags"))
		return flag_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "powlvl") || !strcmpm(front(argument), "powerlevel"))
		return powlvl_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "status"))
		return status_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "chdef") || !strcmpm(front(argument), "default"))
		return default_channel(player, c, restof(argument));
	else if (!strcmpm(front(argument), "owner")) 
		return owner_channel(player, c, restof(argument));
	
	return M_EUNKCMD;
}



int create_channel(player, argument)
	object *player;
	char *argument;
{
	CHANNEL *newc;
	
	if(!*front(argument) || !*restof(argument))
		return M_EINVARG;
	
	if(atoi(atr_get(player, "ChannelQuota")) < 1)
		return M_ECHNOQUOTA;
	
	if( (newc = __find_channel(front(argument))) )
		return M_ECHSHDW;
		
	newc = __create_channel(front(argument), restof(argument));
	newc->oref = player->ref;
	newc->owner = player;
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|Channel |X|%s |C|created.",
		newc->name));

	return TRUE;	
}



int list_channels_on(player, argument)
	object *player;
	char *argument;
{
	CHANNEL *clist;
	object *who;
	
	if(!argument || !*argument)
		who = player;
	else
		if(!(who = match_player(argument)))
			return M_ENOPLAY;
	
	if(!can_control(player, who))
		return M_ENOCNTRL;
		
	if(!who->channels)
		return M_ECHEMPTY;
	
	notify(player, pstr("|X|%s|C|'s |C+|Channel List", color_name(who, -1)));
	for(clist = who->channels; clist; clist = clist->next)
		notify(player, pstr("|%s|%s\t%s", clist->colour, clist->chobj->name,
			(clist->status == CHANNEL_ON) ? "|R+|ON" : "|R+|OFF"));
	
	return TRUE;
}

int list_channels(player, argument)
	object *player;
	char *argument;
{
	CHANNEL *clist;
	
	if(*argument)
		return list_channels_on(player, argument);
		
	if(!channel_list)
		return M_ECHEMPTY;
	
	make_header(player, "|W+|--- |C|List of Channels |W+|---");	
	
	for(clist = channel_list; clist; clist = clist->next)
		notify(player, pstr("|%s|%s|C| is owned by |X|%s",
			clist->colour, clist->name, color_name(clist->owner, -1)));
	
	notify(player, pstr("|B+|%s", make_ln("-", 78)));
	
	return TRUE;
}

int leave_channel(player, argument)
	object *player;
	char *argument;
{
	CHANNEL *channel, *c;
	
	if(!(channel = __find_channel(argument)))
		return M_ENOTFOUND;
	
	if(!(c = __has_channel(player, channel)))
		return M_ECHNOTON;

	__remove_channel(player, c);
	notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|Channel |%s|%s |C|deleted from your list.",
		channel->colour, channel->name));
	notify_channel(NULL, channel, pstr("|R+|* |X|%s |C|has left this channel.",
		color_name(player, -1)));
			
	return TRUE;
}

int join_channel(player, argument)
	object *player;
	char *argument;
{
	CHANNEL *check, *c;
	
	if(!(check = __find_channel(argument)))
		return M_ENOTFOUND;

	if(!parse_lock(player, check->locks) || check->status != CHANNEL_ON)
		return M_ECHNOJOIN;

	if( (c = __has_channel(player, check)))
		return M_ECHALREADY;
	
		
	c = __add_channel(player, check);
	c->chdef = check->chdef;
		
	notify(player, pstr("|%s|%s |C|added to your channel list with alias '|Y+|%s|C|'", 
		c->colour, check->name, check->alias));
	notify_channel(NULL, check, pstr("|R+|* |X|%s|C| has joined this channel.", color_name(player, -1)));
	return TRUE;
}

int view_channel_player(player, arg, channel)
	object *player;
	char *arg;
	CHANNEL *channel;
{
	CHANNEL *show;
	object *who;
	
	if(!(who = (*arg) ?  match_player(arg) : player))
		return M_ENOPLAY;
	
	if(!can_control(player, who))
		return M_ENOCNTRL;
			
	if(!(show = __has_channel(who, channel)))
		return M_ECHNOTFOUND;
	
	notify(player, pstr("|C|Channel statistics for |N+|'|X|%s|N+|'", color_name(who, -1)));
	notify(player, pstr("|M+|Channel Name.|W+|: |%s|%s |W+|[|G+|%s|W+|]", show->colour, show->chobj->name,
		show->alias));
	notify(player, pstr("|B+|Your Title...|W+|: |X|%s", (show->title) ? show->title : "|R+|NONE"));
	notify(player, pstr("|B+|Your Locks...|W+|: |R+|%s", (show->locks) ? show->locks : "NONE"));
	notify(player, pstr("|B+|Colour Code..|W+|: |%s|%s", show->colour, show->colour));
	notify(player, pstr("|B|%s", make_ln("-", 78)));
	notify(player, pstr("|Y+|* |C|Channel is a default channel|W+|: %s",
		(show->chdef == CHANNEL_ON) ? "|Y+|TRUE" : "|R+|FALSE"));

	return TRUE;
}	

int view_channel(player, arg1)
	object *player;	
	char *arg1;
{
	CHANNEL *channel;
	
	if(!*arg1)
		return M_EINVARG;
		
	if(!(channel = __find_channel(arg1)))
		return M_ECHNOTFOUND;
		
	make_header(player, "|W+|--- Channel Information |W+|---");
	notify(player, pstr("\t|B+|Channel Name.........|W+|: |%s|%s |W+|[|G+|%s|W+|]", 
		channel->colour, channel->name, channel->alias));
	notify(player, pstr("\t|B+|Channel Owner........|W+|: |X|%s", color_name(channel->owner, -1)));
	notify(player, pstr("\t|B+|Channel Joinable.....|W+|: %s |W+|(|G+|%s|W+|)", (parse_lock(player, channel->locks)) ?
		"|Y+|YES" : "|R+|NO", (channel->locks) ? channel->locks: "NONE"));
	notify(player, pstr("\t|B+|Channel Title........|W+|: |X|%s", channel->title));
	notify(player, pstr("|B|%s", make_ln("-", 78)));
	notify(player, pstr("\t%s", channel->desc));
	notify(player, pstr("|B+|%s", make_ln("=", 78)));
	return FALSE;
}

int channel_to(player, channel, pchannel, argument)
	object *player;
	CHANNEL *channel;
	CHANNEL *pchannel;
	char *argument;
{
	char buf[MAX_BUFSIZE];
	object *o;
	CHANNEL *c;
	
	if(!(o = match_player(front(argument)))) 
		return M_ENOPLAY;

	if(!(c = __has_channel(o, channel)))
		return M_ENOPLAY;
	
	if(!parse_lock(player, c->locks) || !parse_lock(o, pchannel->locks))
		return M_ECHLOCPLY;
		
	sprintf(buf, "|X|%s |W+|[|Y+|to|X| %s|W+|]|X| %s", color_name(player, -1), color_name(o, -1),
		speech_parse(player, restof(argument)));
	notify_channel(pchannel, channel, buf);
	return TRUE;
}

char *channeltitle(pchannel)
	CHANNEL *pchannel;
{
	char buf[MIN_BUFSIZE];
	
	if(pchannel->title)
		sprintf(buf, " |M+|(|X|%s|M+|)", pchannel->title);
	else
		return "";
	
	return stralloc(buf);
}

int send_to_channel(player, channel, pchannel, argument)
	object *player;
	CHANNEL *channel;
	CHANNEL *pchannel;
	char *argument;
{
	char buf[MAX_BUFSIZE];
	
	if(*argument == POSE_TOKEN)
		sprintf(buf, "|X|%s|X| %s", color_name(player, -1), argument + 1);
	else if (*argument == POSS_TOKEN)
		sprintf(buf, "|X|%s|X|'s %s", color_name(player, -1), argument + 1);
	else if (*argument == THINK_TOKEN)
		sprintf(buf, "|X|%s |N+|. |B|o |B+|O |C+|(|X| %s|C+| )", color_name(player, -1),
			argument + 1);
	else if (*argument == TO_TOKEN)
		return channel_to(player, channel, pchannel, argument + 1);
	else
		sprintf(buf, "|X|%s%s|W+|: |X|%s", color_name(player, -1), channeltitle(pchannel), argument);
	
	notify_channel(pchannel, channel, buf);
	return TRUE;
}

int turn_channel(player, channel, flag)
	object *player;
	CHANNEL *channel;
	int flag;
{
	CHANNEL *pchannel;
	
	if(!(pchannel = __has_channel(player, channel)))
		return M_ECHNOTFOUND;
	
	if(flag == CHANNEL_OFF) {
		if(pchannel->status == CHANNEL_OFF) {
			notify(player, "|Y+|* |C|Channel is already off.");
			return TRUE;
		} else {
			notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|You turned |%s|%s |R+|OFF",
				pchannel->colour, channel->name));
			pchannel->status = CHANNEL_OFF;
			notify_channel(pchannel, channel, pstr("|R+|* |X|%s |C|has turned this channel off.",
				color_name(player, -1)));
			return TRUE;
		}
	} else if (flag == CHANNEL_ON) {
		if(pchannel->status == CHANNEL_ON) {
			notify(player, "|Y+|* |C|Channel is already on.");
			return TRUE;
		} else {
			notify(player, pstr("|C+|[|Y+|CHANNEL|C+|]|W+|: |C|You turned |%s|%s |Y+|ON",
				pchannel->colour, channel->name));
			pchannel->status = CHANNEL_ON;
			notify_channel(pchannel, channel, pstr("|R+|* |X|%s |C|has turned this channel on.",
				color_name(player, -1)));
			return TRUE;
		}
	} 
	
	return M_EUNKCMD;
}

int player_default_channel(player, channel)
	object *player;
	CHANNEL *channel;
{
	CHANNEL *clist;
	
	for(clist = player->channels; clist; clist = clist->next)
		if(clist == channel)
			clist->chdef = TRUE;
		else
			clist->chdef = FALSE;
	
	notify(player, pstr("|%s|%s |C|is now your default channel.", channel->colour, channel->name));
	return TRUE;	
}

int player_channel_set(player, channel, argument)
	object *player;
	CHANNEL *channel;
	char *argument;
{
	CHANNEL *pchannel;
	
	if(!(pchannel = __has_channel(player, channel)))
		return M_ECHNOTFOUND;
	
	if(!strcmpm(front(argument), "lock"))
		return lock_channel(player, pchannel, restof(argument));
	else if (!strcmpm(front(argument), "alias"))
		return player_alias_channel(player, pchannel, restof(argument));
	else if (!strcmpm(front(argument), "title"))
		return title_channel(player, pchannel, restof(argument));
	else if (!strcmpm(front(argument), "colour") || !strcmpm(front(argument), "color"))
		return colour_channel(player, pchannel, restof(argument));
	else if (!strcmpm(front(argument), "default") || !strcmpm(front(argument), "chdef"))
		return player_default_channel(player, pchannel);
	else {
		if(pchannel->status == CHANNEL_ON && channel->status == CHANNEL_ON)
			return send_to_channel(player, channel, pchannel, argument); 
		else
			return M_ECHNOTON;
	}
}

int who_channel(player, channel)
	object *player;
	CHANNEL *channel;
{
	CHANNEL *c, *cl;
	OBJECT *o;
	int ctr = 0;
	
	if(!(c = __has_channel(player, channel)))
		return M_ECHNOTON;

	if(c->status == CHANNEL_OFF)
		return M_ECHNOTON;
		
	notify(player, pstr("|C|Channel |W+|%s |C|who list|W+|:", c->name));
	notify(player, pstr("|B|%s", make_ln("-",strlen(c->name) + 17)));
	for(o = player_list; o; o=o->next)
		if((o->flags & CONNECT) && ( !(o->flags & INVISIBLE) || ((o->flags & INVISIBLE) && (player->flags & WIZARD))))
			for(cl = o->channels; cl; cl=cl->next)
				if(cl->chobj == channel) {
					notify(player, pstr("%s %s", color_name(o, -1), (o->flags & INVISIBLE) ?
						"|W+|(INVISIBLE)" : ""));
					ctr++;
				}
	notify(player, "|W+|-----");
	notify(player, pstr("|C|Total Players|W+|: |M+|%d", ctr));
	return TRUE;	
}



int do_channel_event(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	CHANNEL *channel;
	char argument[MAX_BUFSIZE];
	
	strcpy(argument, arg2);
	if(!(channel = __find_channel(arg1)))
		return M_ECHNOTFOUND;

	if(!*argument)
		return view_channel_player(player, channel);
	
	if (!strcmpm(front(argument), "on") && !*restof(argument))
		return turn_channel(player, channel, CHANNEL_ON);
	else if (!strcmpm(front(argument), "boot")) 
		return boot_channel(player, channel, restof(argument));
	else if (!strcmpm(front(argument), "off") && !*restof(argument))
		return turn_channel(player, channel, CHANNEL_OFF);
	else if (!strcmpm(front(argument), "who") && !*restof(argument))
		return who_channel(player, channel);
	else if (!strcmpm(front(argument), "view")) 
		return view_channel_player(player, restof(argument), channel);
	else if (!strcmpm(front(argument), "paste") && !*restof(argument))  {
		sprintf(argument, "paste channel %s", channel->name);
		filter_command(player, argument);
		return TRUE;	
	} else
		return player_channel_set(player, channel, argument);
}

int delete_channel(player, chan)
	object *player;
	char *chan;
{
	CHANNEL *channel;
	
	if(!(channel = __find_channel(chan)))
		return M_ECHNOTFOUND;
	
	if(!channel_control(player, channel))
		return M_ENOCNTRL;
	
	SET_ATR(player, "ChannelQuota", pstr("%d", atoi(atr_get(player, "ChannelQuota")) + 1));
	__purge_channel(channel);
	__delete_channel(channel);
	notify(player, "|Y+|* |C|Channel deleted.");
	return TRUE;
		
}

void do_channel(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	if(!*arg1) { // list the channels player is on.
		error_report(player, list_channels_on(player, NULL));
		return;
	}
	
	if (!strcmpm(arg1, "list")) {
		error_report(player, list_channels(player, arg2));
		return;
	} else if(!strcmpm(arg1, "create")) {
		error_report(player, create_channel(player, arg2));
		return;
	} else if (!strcmpm(arg1, "leave")) {
		error_report(player, leave_channel(player, arg2));
		return;
	} else if(!strcmpm(arg1, "join")) {
		error_report(player, join_channel(player, arg2));
		return;
	} else if (!strcmpm(arg1, "set")) {
		error_report(player, do_channel_set(player, front(arg2), restof(arg2)));
		return;
	} else if (!strcmpm(arg1, "view")) {
		error_report(player, view_channel(player, front(arg2), restof(arg2)));
		return;
	} else if (!strcmpm(arg1, "delete")) {
		error_report(player, delete_channel(player, arg2));
		return;
	} else {
		error_report(player, do_channel_event(player, arg1, arg2));
		return;
	}
}

int match_channels(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	CHANNEL *clist;
	
	for(clist = player->channels; clist; clist=clist->next)
		if(!strcmpm(clist->alias, arg1)) {
			error_report(player, do_channel_event(player, clist->name, arg2));
			return 1;
		}
	
	return 0;
}

void do_channel_default(player, sendstr)
	object *player;
	char *sendstr;
{
	CHANNEL *clist;
	
	for(clist = player->channels; clist; clist=clist->next)
		if(clist->chdef) {
			error_report(player, do_channel_event(player, clist->name, sendstr));
			return;
		}
	
	error_report(player, M_ECHNODFLT);
}

/** CHANNEL PASTE **/
int channel_paste_header(d, channame, tag)
	DESCRIPTOR *d;
	char *channame;
	int tag;
{
	CHANNEL *channel, *pchannel;
	
	if(!(channel = __find_channel(channame)))
		return M_ECHNOTFOUND;
	
	if(!(pchannel = __has_channel(d->player, channel)))
		return M_ECHNOTON;
	
	if(tag)
		notify_channel(pchannel, channel, pstr("|C+|--- |C|Paste Buffer From |X|%s |C+|---",
			color_name(d->player, -1)));
	else
		notify_channel(pchannel, channel, pstr("|C+|--- |C|End of Paste Buffer From |X|%s |C+|---",
			color_name(d->player, -1)));	
	return TRUE;
}

void paste_channel(player, channame, message)
	object *player;
	char *channame;
	char *message;
{
	CHANNEL *channel, *pchannel;
	
	if(!(channel = __find_channel(channame)))
		return;
	
	if(!(pchannel = __has_channel(player, channel)))
		return;
	
	notify_channel(pchannel, channel, message);
}

/** CHANNEL LOGS **/
void log_channel(channame, message)
	char *channame;
	char *message;
{
	char buf[MAX_BUFSIZE];
	CHANNEL *channel;
	FILE *fp;
	
	if(!(channel = __find_channel(channame)))
		return;
	
	sprintf(buf, "logs/%s", channame);
	
	if(!(fp = fopen(buf, "a"))) {
		notify_channel(NULL, channel, "|R+|* |C|Could not open log!");
		return;
	}
	
	notify_channel(NULL, channel, message);
	
	fprintf(fp, "(%s)[%s] %s\n", get_date(time(0)), channel->name, strip_color(message));
	fclose(fp);
}