/****************************************************************************
 MYTH - Revised Attribute System
 Coded by Saruk (04/05/99)
 ---
 This is a new revised attribute system. Please be advised that 'player' refers
 in many cases to any object.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

// Delete an attribute from a player's attribute list.
void __delete_attribute(player, atr)
	object *player;
	attr *atr;
{
	attr *a, *aprev = NULL;
	
	for(a = player->alist; a; a=a->next) {
		if(a == atr)
			break;
		aprev = a;
	}
	
	if(!a)
		return;
	
	if(!aprev)
		player->alist = a->next;
	else
		aprev->next = a->next;
	
	if(a->name)
		free(a->name);
	if(a->value)
		free(a->value);
	
	free(a);
}

void KILL_ATTRS(player)
	object *player;
{
	attr *a, *anext;
	
	for(a = player->alist; a; a=anext) {
		anext = a->next;
		__delete_attribute(player, a);
	}
	
	player->alist = NULL;
}
	
// Add an attribute to a player's attribute list
// Returns the new attribute's memory reference.
ATTR *__add_attribute(player, atr)
	object *player;
	attr *atr;
{
	ATTR *anew;
	
	GENERATE(anew, ATTR);
	anew->name = NULL;
	SET(&anew->name, atr->name);
	anew->value = NULL;
	anew->flags = atr->flags;
	anew->powlvl = atr->powlvl;
	
	anew->next = player->alist;
	player->alist = anew;
	
	return(anew);			
}

// Searches for a player's attribute - returns NULL if attribute is not found.
ATTR *__has_attribute(player, atrname)
	object *player;
	char *atrname;
{
	attr *a;
	
	for(a = player->alist; a; a=a->next)
		if(!strcmpm(a->name, atrname) && a->value)
			return a;
	
	return (NULL);
}

ATTR *__find_default_attribute(atrname)
	char *atrname;
{
	int ctr;
	attr *a;
	
	for(ctr = 0; myth_defined[ctr].name; ctr++) {
		a = &myth_defined[ctr];
		if(!strcmpm(a->name, atrname)) 
			return(a);
	}
	
	return (NULL);
}

int __is_attribute(atrname)
	char *atrname;
{
	attr *a;
	
	if(!(a = __find_default_attribute(atrname)))
		return 0;
		
	return 1;
}

/* Checks to see if alias is someone ELSE. */
int __check_alias(player, value)
	object *player;
	char *value;
{
	object *o;

	if(ISNUM(match_num(value)))
		return 0;
		
	if(!(o = match_player(strip_color(value))))
		return 1;
	
	if(o == player)
		return 1;
	
	return 0;
}

int __check_attribute(a, value)
	attr *a;
	char *value;
{
	object *o;
	attr *atr;
	
	for(o = player_list; o; o=o->next)
		if( (atr = __has_attribute(o, a->name)) )
			if(!strcmpm(strip_color(atr->value), strip_color(value)) && atr != a)
				return 0;
	
	return 1;
}


char *atr_get(player, atrname)
	object *player;
	char *atrname;
{
	attr *a;
	
	if(!(a = __has_attribute(player, atrname)))
		if(!(a = __find_default_attribute(atrname)))
			return "";

	if(!a->value)
		return "";
		
	return a->value;		
}

/* 
   This is a "no holds barred" attribute setter. It doesn't do more than check to make sure the
   attribute exists.
*/
void SET_ATR(player, atrname, value)
	object *player;
	char *atrname;
	char *value;
{
	attr *a;
	
	if(!(a = __has_attribute(player, atrname))) {
		if(!strcmpm(value, "NULL"))
			return;
		if(!(a = __find_default_attribute(atrname)))
			return;
		a = __add_attribute(player, a);
	}
	
	if(!strcmp(value, "NULL"))
		a->value = NULL;
	else
		SET(&a->value, value);
}

/** SETTING AN ATTRIBUTE **/
// This function will set an attribute's value, or if that attribute has not been added
// to the attribute list, it will add that attribute and then set it's value.
// ** This is the universal attribute set function called by do_set(). It performs checks
// ** for several different states.
int atr_set(setter, player, atrname, value)
	object *setter;
	object *player;
	char *atrname;
	char *value;
{
	attr *a;
	
	if(player->flags & LOCKED)
		return M_EISLOCKED;
				
	if(!(a = __has_attribute(player, atrname))) {
		if(!(a = __find_default_attribute(atrname)))
			return M_EILLATTR;
		a = __add_attribute(player, a);
	}
	

	
	if( (a->flags & AWIZARD) && !(setter->flags & WIZARD)) 
		return M_ENOCNTRL;
	
	if( (a->powlvl > setter->powlvl) ) 
		return M_EBADPOW;
		
	if( (a->flags & ACHECK) )
		if(!__check_attribute(a, value))
			return M_ENOCANDO;
	
	if(!strcmpm(value, "NULL")) {
		__delete_attribute(player, a);
		return 1;
	}		
	
	if(!strcmpm(atrname, "alias"))
		if(!__check_alias(player, value))
			return M_ENOCANDO;
				
	SET(&a->value, value);
	return 1;
}

// Check attributes and delete those that do not belong anymore.
void check_attrs(player)
	object *player;
{
	attr *a, *check, *anext = NULL;
	int ctr;
	
	for(ctr = 0; myth_defined[ctr].name; ctr++)
		for(a = player->alist; a; a=anext) {
			anext = a->next;
			if(!(check = __find_default_attribute(a->name)))
				__delete_attribute(player, a);
		}
}

void atr_examine(player, victim, atrname)
	object *player;
	object *victim;
	char *atrname;
{
	attr *a;
	
	if(!(a = __has_attribute(victim, atrname)))
		if(!(a = __find_default_attribute(atrname))) {
			error_report(player, M_EILLATTR);
			return;
		}

	if( ( (a->flags & AWIZARD) && (a->flags & ASEE) ) || ( ((a->flags & AWIZARD) && !(a->flags & ASEE)) &&
		(player->flags & WIZARD)) || !(a->flags & AWIZARD)) 
		notify(player, pstr("|B+|%s|W+|(|Y+|%s|W+|):|C| %s", a->name, aflaglist(a->flags),
			(a->value) ? a->value : "|R+|Empty!"));
	else
		notify(player, "|Y+|* |R+|You can't see that!");
}

void mkattr(player, instr)
	object *player;
	char *instr;
{
	ATTR *a;
	char buf[MAX_BUFSIZE];
	char parsed[4][MAX_BUFSIZE];
	
	strcpy(buf, instr);
	strcpy(parsed[0], fparse(buf, ':')); // owner
	strcpy(buf, rparse(buf, ':'));
	strcpy(parsed[1], fparse(buf, ':')); // flags
	strcpy(buf, rparse(buf, ':'));
	strcpy(parsed[2], fparse(buf, ':')); // name
	strcpy(parsed[3], rparse(buf, ':')); // value

	if(!(a = __find_default_attribute(parsed[2])))
		return;
	
	a = __add_attribute(player, a);
	a->powlvl = atoi(parsed[0]);
	a->flags = atoi(parsed[1]);
	SET(&a->value, parsed[3]);

}