/*****************************************************************************
 MYTH - Matching Arguments 
 Coded by Saruk (04/25/99)
 ---
 Match everything
 *****************************************************************************/
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

/* this matches a partial player WITHIN a location. */
object *match_partial_type(location, matchname, flag)
	object *location;
	char *matchname;
	int flag;
{
	CONTENTS *c;
	int ref;
	char buf[MAX_BUFSIZE];
	
	if(!matchname || !*matchname)
		return (NULL);
	
	ref = match_num(matchname);
		
	for(c = location->contents; c; c=c->next) {
		strcpy(buf, c->content->name);
		while(*buf) {
			if( (!strcmpm(strip_color(atr_get(c->content, "Alias")), matchname) || 
				string_prefix(matchname, buf) || ref == c->content->ref) && 
					(c->content->flags & flag))
				return c->content;
			strcpy(buf, restof(buf));
		}
	}
	
	return (NULL);		
}

int match_num(char *str)
{
	static char buf[MAX_BUFSIZE];
	char *p = buf;
	
	if(*str == '-') 
		*p++ = *str, str++;
		
	while(*str && isdigit(*str))
		*p++ = *str, str++;
	
	if(*str)
		return -1;
	
	*p++ = '\0';
	return atoi(buf);
}

// Match a player's name.
object *match_player(matchname)
	char *matchname;
{
	object *o;
	int ref;	// if char is a number value, we want to return this as well.
	
	if(!*matchname)
		return (NULL);

	// let's check to see if it really a 0 or not.
	ref = match_num(matchname);
	
	for(o=player_list; o; o=o->next)
		if(!strcmpm(o->name, matchname) || ref == o->ref || !strcmpm(strip_color(atr_get(o, "Alias")), matchname))
			return o;
	
	return (NULL);
}

// Match a room's name.
object *match_room(matchname)
	char *matchname;
{
	object *o;
	int ref;
	
	if(!*matchname)
		return (NULL);
		
	ref = match_num(matchname);
	for(o = room_list; o; o=o->next) 
		if(!strcmpm(o->name, matchname) || ref == o->ref)
			return o;
	
	return (NULL);
}

// Match a thing.
object *match_thing(matchname)
	char *matchname;
{
	object *o;
	int ref;	// if char is a number value, we want to return this as well.
	
	if(!*matchname)
		return (NULL);
	
	ref = match_num(matchname);	
	for(o=thing_list; o; o=o->next)
		if(!strcmpm(o->name, matchname) || ref == o->ref)
			return o;
	
	return (NULL);
}

int match_type(victim, type)
	object *victim;
	char *type;
{
	int ctr = 0;
	
	for(ctr = 0; flag_table[ctr].name; ctr++)
		if((string_prefix(type, flag_table[ctr].name) || *type == flag_table[ctr].parsed) &&
			(victim->flags & flag_table[ctr].flag))
			return 1;
	
	return 0;
}

object *match_special(player, matchname)
	object *player;
	char *matchname;
{
	if(!*matchname)
		return (NULL);

	if(!strcmpm(matchname, "me") || !strcmpm(matchname, "self"))
		return player;
	else if (!strcmpm(matchname, "here"))
		return player->location;
	
	return (NULL);
}

object *match_partial(location, matchname)
	object *location;
	char *matchname;
{
	char buf[MAX_BUFSIZE];
	CONTENTS *cont;
	
	if(!matchname || !*matchname)
		return (NULL);
		
	for(cont = location->contents; cont; cont=cont->next) {
		strcpy(buf, cont->content->name);
		while(*buf) {
			if(string_prefix(matchname, buf))
				return cont->content;
			strcpy(buf, restof(buf));
		}
	}
	
	return (NULL);
}

object *match_moveable(matchname)
	char *matchname;
{
	object *o;
	
	if(!*matchname)
		return (NULL);
	
	if(!(o = match_player(matchname)))
		if(!(o = match_thing(matchname)))
			return (NULL);
	
	return o;
}

object *match_exit(matchname)
	char *matchname;
{
	object *o;
	int ref;
	
	if(!*matchname)
		return (NULL);
	ref = match_num(matchname);
	
	for(o=exit_list; o; o=o->next)
		if(!strcmpm(o->name, matchname) || ref == o->ref)
			return o;
	
	return (NULL);
}

// Match with all objects.
object *match_object(matchname)
	char *matchname;
{
	object *o;
	
	if(!*matchname)
		return (NULL);
	if ( (o=match_player(matchname)) )
		return o;
	
	if( (o=match_room(matchname)) )
		return o;
				
	if( (o=match_thing(matchname)) )
		return o;

	if( (o = match_exit(matchname)) )
		return o;		
			
	return (NULL);	
}
