/****************************************************************************
 MYTH - Object handling.
 Coded by Saruk (02/08/99)
 ---
 Handle objects routines such as content adding, manipulation of locations, 
 etc.
 ****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "db.h"
#include "net.h"
#include "externs.h"

// Add contents of an object.
void add_content(player, what)
	object *player;
	object *what;
{
	CONTENTS *c;
	
	GENERATE(c, CONTENTS);
	c->content = what;
	c->next = player->contents;
	player->contents = c;
}

// Delete contents of an object.
void del_content(player, what)
	object *player;
	object *what;
{
	CONTENTS *c, *cprev = NULL;
	
	for(c = player->contents; c; c=c->next) {
		if(c->content == what) 
			break;
		cprev = c;
	}
	
	if(!c)
		return;
	
	if(!cprev)
		player->contents = c->next;
	else
		cprev->next = c->next;
	
	free(c);
}			

// This will use the move_to procedure and move an object to an object.
void do_teleport(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	object *victim, *where = NULL;
	
	IS_ARGUMENT(player, (arg1));
	
	if(!(victim = match_object(arg1))) {
		error_report(player, M_EILLNAME);
		return;
	}
	
	if(*arg2) {
		if(!(where = match_object(arg2))) {
			error_report(player, M_EILLNAME);
			return;
		}
	}

		
	if(!where) {
		if(player->location == victim) {
			notify(player, "|Y+|* |C|You're already at this location!");
			return;
		}
		if( (victim->flags & CAN_TEL) || can_control(player, victim) || player->linkto == victim) {
			notify(player, pstr("|C+|[|Y+|TELEPORT|C+|]|W+|: |C|Teleporting you to |X|%s",
				color_name(victim, -1)));
			move_to(player, victim);
		} else 
			error_report(player, M_ENOCNTRL);
	} else {
		if( !(player->flags & WIZARD) && (victim->flags & WIZARD)) {
			error_report(player, M_ENOCNTRL);
			return;
		}
		if ( (can_control(player, victim) || (victim->location == player->location && 
			player->location->ownit == player)) && (can_control(player, where) || (where->flags & CAN_TEL) ||
				(where == victim->linkto))) {
				if(victim->location == where) {
					notify(player, pstr("|Y+|*|X| %s|C| is already at that location!", color_name(victim, -1)));
					return;
				}
			notify(player, pstr("|C+|[|Y+|TELEPORT|C+|]|W+|: |C|Teleporting |X|%s |C|to |X|%s",
				color_name(victim, -1), color_name(where, -1)));
			move_to(victim, where);
			notify(victim->ownit, pstr("|C|[|Y+|TELEPORT|C+|]|W+|: |X|%s |C|moved |X|%s to |X|%s",
				(victim->ownit == player) ? "|C|You":color_name(player, -1), 
					(victim->ownit == victim) ? "|C|you": color_name(victim, -1),
						color_name(where, -1)));
		} else {
			error_report(player, M_ENOCNTRL);
		}
	}
}

// Return a string of flags. Like "PWc" (Player Wizard connect)
char *flaglist(player)
	object *player;
{
	char buf[MAX_BUFSIZE];
	char *p = buf;
	int ctr;
	
	for(ctr = 0; flag_table[ctr].name; ctr++) 
		if( (player->flags & flag_table[ctr].flag) && (player->flags & flag_table[ctr].type))
			if(!(flag_table[ctr].flag & SUSPECT))
				*p++ = flag_table[ctr].parsed;	
	
	*p++ = '\0';
	return stralloc(buf);
}

char *aflaglist(aflags)
	int aflags;
{
	char buf[MAX_BUFSIZE];
	char *p = buf;
	int ctr;
	
	for(ctr = 0; aflag_table[ctr].name; ctr++)
		if( (aflags & aflag_table[ctr].flag) )
			*p++ = aflag_table[ctr].parsed;
	
	*p++ = '\0';
	return stralloc(buf);
}

void do_take(player, str)
	object *player;
	char *str;
{
	object *thing;
	
	IS_ARGUMENT(player, (str));
	
	if(!(thing = match_moveable(str))) 
		if(!(thing = match_partial(player->location, str))) {
			error_report(player, M_EILLNAME);
			return;
		}
	
	if(thing->location != player->location) {
		error_report(player, M_EREMOTE);
		return;
	}
	
	take_trigger(player, thing);
}

void do_drop(player, str)
	object *player;
	char *str;
{
	object *thing;

	IS_ARGUMENT(player, (str));
	
	if(!(thing = match_moveable(str))) 	
		if(!(thing = match_partial(player, str))) {
			error_report(player, M_EILLNAME);
			return;
		}
	
	if(thing->location != player) {
		error_report(player, M_ENOTHELD);
		return;
	}
	
	move_to(thing, player->location);
	notify(player, pstr("|C+|[|Y+|DROP|C+|]|W+|: |C|You dropped|X| %s|C|...",
		color_name(thing, -1)));
}

char *unparse_object(victim)
	object *victim;
{
	char buf[MAX_BUFSIZE];
	
	sprintf(buf, "|X|%s |W+|(|Y+|%d%s|W+|)", color_name(victim, -1), victim->ref, flaglist(victim));
	
	return stralloc(buf);
}

int change_ownership(player, victim, newvalue)
	object *player;
	object *victim;
	char *newvalue;
{
	object *newowner;
	char buf[MAX_BUFSIZE];
	
	if(!(newowner = match_player(newvalue)))
		return M_ENOPLAY;

	if(victim->flags & TYPE_PLAYER)
		return M_ENOCANDO;
	
	if(!can_control(player, newowner))
		return M_ENOCNTRL;
			
	victim->owner = newowner->ref;
	victim->ownit = newowner;
	strcpy(buf, color_name(newowner, -1));
	log_channel("log_sens", pstr("|R+|* |C|Owner of |X|%s |C|changed to|W+|: |X|%s",
		unparse_object(victim), color_name(newowner, -1)));
	notify(player, pstr("|C+|[|Y+|BUILTIN|C+|]|W+|:|C| Owner of|X| %s|C| changed to |X|%s|C|...",
		unparse_object(victim), buf));
		
	return 1;
}


int change_home(player, victim, newvalue)
	object *player;
	object *victim;
	char *newvalue;
{
	object *newlink;
	char buf[MAX_BUFSIZE];
	
	if(!(newlink = match_object(newvalue)))
		if(!(newlink = match_special(player, newvalue)))
			return M_ENOPLAY;
	
	if(newlink->flags & TYPE_EXIT)
		return M_EBADLNK;
		
	if(!can_control(player, newlink))
		return M_ENOCNTRL;
	
	victim->link = newlink->ref;
	victim->linkto = newlink;
	strcpy(buf, color_name(newlink, -1));
	notify(player, pstr("|C+|[|Y+|BUILTIN|C+|]|W+|:|C| %s of |X|%s|C| changed to |X|%s|C|...",
		(victim->flags & TYPE_PLAYER) ? "Home":"Link", unparse_object(victim), buf));

	return 1;	
}