/****************************************************************************
 MYTH - Database Headers.
 Coded by Saruk (01/12/99)
 ---
 General includes, such as the database and configuration definitions.
 ****************************************************************************/
// GNU C Includes
#include <stdio.h>
#include <sys/time.h>

#define _XOPEN_SOURCE
#include <unistd.h>

// General MYTH configuration definitions.
#include "memory.h"
#include "config.h"

// Flags, errors, mail definitions, attribute table definitions.
// NOTE: Some includes are appended to this file. Some use object_t.
#include "locks.h"
#include "flags.h"
#include "error.h"

typedef struct object_t object;
typedef struct object_t OBJECT;

// Object heap file.
#define DEAD_OBJ_HEAP "deadobj.db"

// Initial power settings.
#define PLAYER_POWINIT	25
#define	THING_POWINIT	10
#define ROOM_POWINIT	15
#define EXIT_POWINIT	5

// Object attribute definitions.
typedef struct attr_typ {
	char *name;
	char *value;
	int flags;
	int powlvl;
	
	struct attr_typ *next;
} attr, attr_list, ATTR, ATTR_LIST;

// Object command list structure. (for aliasing)
typedef struct clist_t {
	char *cmd;
	char *send;
	
	struct clist_t *next;
} CLIST;



// Content lists for objects.
typedef struct content_t {
	OBJECT *content;
	
	struct content_t *next;
} CONTENTS;

/** MAIL SYSTEM **/
#include "mail.h"

/** CHANNEL SYSTEM **/
#include "channels.h"

/** Bulletin Board System **/
typedef struct bboard_t {
	char *topic;		// Topic/Title
	int c_ref;		// creator reference
	object *creator;
	char *message;
	time_t created;
	time_t expiration;	
	int num_read;
	
	struct bboard_t *next;
} BBOARD;

	
// The object type. ( ** Our main database structure ** )
typedef struct object_t {
	char *name;		// name of object.
	char *desc;		// description of object.
	char *pass;		// password of object.
	dbref ref;		// database reference number.
	dbref owner;		// owner priveleges of object.
	dbref loc;		// location of object.
	dbref link;		// this points to the home of an object.
	int powlvl;		// power level of object (probably changing)
	long flags;		// object flags.

	// Pointers...
	ATTR *alist;		// Attribute list.
	MAIL *mail;		// Our mail system
	CLIST *cmdlist;		// object command list.
	CHANNEL *channels;	// channel list.
	CONTENTS *contents;	// a list of objects in contents.
	
	// pointers to objects.
	struct object_t *ownit;
	struct object_t *location;
	struct object_t *linkto;

	// pointer to next object in list. 
	struct object_t *next;
} OBJ;

// Assign external/global object lists.
extern object *exit_list;
extern object *thing_list;
extern object *room_list;
extern object *player_list;
extern object *rootobj;
extern ATTR_LIST myth_defined[];
extern BBOARD *bblist;