/****************************************************************************
 MYTH - (Input/Output, Basic engine)
 Coded by Saruk (01/10/99)
 ---
 This is the main game engine for MYTH. It was initially started January 10th, 
 1999 - though there were attempts made as far back as Late Spring/Early Summer
 of 1994.
 ****************************************************************************/
/** General Includes **/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <arpa/telnet.h>

/** MYTH Includes **/
#include "db.h"
#include "net.h"
#include "externs.h"

// Local Definitions
#define DEF_MODE	0644
#define MAX_PID		65535

// Global variables
int myth_reboots	= 0;
int shutdown_flag	= 0;
int reboot_style	= 0;
int wizlock		= 0;
int sock		= 0;
int resock		= 0;
int maxd		= 0;
int reboot_flag		= 0;
int ndescriptors	= 0;
int num_mallocs		= 0;
int num_frees		= 0;

long total_input	= 0;
long total_output	= 0;
long myth_uptime	= 0;

time_t myth_reboot	= 0;
time_t now		= 0;

size_t tot_mem_act	= 0;

DESCRIPTOR *descriptor_list = 0;

#include "sigtable.h"
#include "ioheader.h"


int main(argc, argv)
	int argc;
	char *argv[];
{
	setup_hooks();
	if(!load_config())
		argv[1] = "-config";
		
	parse_cmd(argv);
	
	redirect_io();
	set_signals();
	
	printf("=========== log.it ============\n");
	printf("Start Time: %s\n", get_date(time(0)));
	myth_uptime = time(0);
	myth_reboot = time(0);
	myth_reboots = 0;

	open_sockets();
	load_database();
	attach_players();
	load_mail();
	load_channel_database();
	softcode_startup();
	
	report_online();
	io_loop();

	close_sockets();
	save_database(return_config("db_name"));
	save_mail(return_config("maildb"));
	save_channel_database(return_config("comdb"));
		
	if (reboot_flag) 
		fcntl(sock, F_SETFD, 1);
	else 
		close(sock);
	
	if (reboot_flag) {
		execv(argv[0], argv);
		unlink("socket_table");
	}

#ifdef CATCHSEG
	unlink("panic_list");	
#endif /*CATCHSEG*/
	return 1; // we're outta here.
}

void process_output(d)
	DESCRIPTOR *d;
{
	TQUEUE *t;
	char buf[MAX_BUFSIZE];
	int colour_tag = 1;
	
	if(d->output_lines > MAX_OUTPUT) {
		sprintf(buf, "|Y+|*** |R+|OUTPUT FLUSHED |Y+|***\n");
		write(d->descriptor, colourize(buf), strlen(colourize(buf)));
	}
	
	if(d->player)
		if(!(d->player->flags & ANSI)) {
			colour_tag = 0;
		}

	for(t = d->output_queue; t; t=t->next)
		if((d->output_lines--) < MAX_OUTPUT) {
			if(colour_tag)
				write(d->descriptor, colourize(t->buffer), strlen(colourize(t->buffer)));
			else
				write(d->descriptor, strip_color(t->buffer), strlen(strip_color(t->buffer)));
		}
		
	d->output_queue = TQUEUE_FREE(d->output_queue);
	d->output_lines = 0;
	d->output_flag = 0;	
}

void io_loop(void)
{
	int errval;
	int available_descriptors;
	fd_set input_set, output_set;
	struct timeval timeout;
	struct descriptor_data *d, *newd, *dnext = NULL;

	sock = make_socket(atoi(return_config("port")));
	if(maxd <= sock)
		maxd = sock + 1;
	
	available_descriptors = getdtablesize() - 5;

	while(!shutdown_flag) {
		clear_stack();
		process_commands();
		if(shutdown_flag)
			break;
			
		do_timer();
			
		for(d=descriptor_list; d; d=d->next)
			if(d->output_flag)
				process_output(d);
				
		FD_ZERO(&input_set);
		FD_ZERO(&output_set);
		
		if(sock > -1)
			FD_SET(sock, &input_set);
		
		for(d=descriptor_list; d; d=d->next)
			if(d->output_flag)
				FD_SET(d->descriptor, &output_set);
			else if (!d->input_flag)
				FD_SET(d->descriptor, &input_set);
				
		for(d=descriptor_list; d; d=d->next)
			if(FD_ISSET(d->descriptor, &output_set))
				if(d->output_flag)
					process_output(d);
			
		timeout.tv_sec = 1;
		timeout.tv_usec = 1;
		
		if ( (errval = select(maxd, &input_set, &output_set, NULL, &timeout)) < 0) {
			printf("%s\n", strerror(errval));
			return;
		} else {
			if(FD_ISSET(sock, &input_set)) {
				if(!(newd = new_connection(sock))) {
					printf("error in new connection!\n");
				} else {
					if(newd->descriptor >= maxd)
						maxd = newd->descriptor + 1;
				}
			}
		}

		for(d=descriptor_list; d; d=dnext) {
			dnext = d->next;
			if(FD_ISSET(d->descriptor, &input_set))
				if (!d->input_flag)
					if(!process_input(d))
						dfree(d->descriptor);
		}
	}
}

static int make_socket(port)
	int port;
{
	int s, opt;
	int errval;
	struct sockaddr_in server;
	int serversize = sizeof(server);
	
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s < 0) {
		printf("error creating socket!\n");
		shutdown_flag = 1;
		return -1;
	}
	
	opt = 1;
	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &opt, sizeof(opt)) < 0) {
		printf("error in setsockopt!\n");
		shutdown_flag = 1;
		close(s);
		return -1;
	}
	
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = INADDR_ANY;
	server.sin_port = htons(port);
	
	if ( (errval = bind(s, (struct sockaddr *)&server, serversize)) !=0) {
		printf("bind(): %s\n", strerror(errval));
		resock = 1;
		close(s);
		return -1;
	}
	
	resock = 0;
	listen(s, 5);
	return s;
}


void close_sockets(void)
{
	DESCRIPTOR *d;
	FILE *f;
	char buf[1024];

	if(!reboot_flag || sock < 0)
		return;

#ifdef CATCHSEG
	/* Obviously rebooting properly, kill the panic_list. */
	unlink("panic_list");
#endif /*CATCHSEG*/
	
	unlink("socket_table");
	if (!(f = fopen("socket_table", "w")))
		return;
	
	fprintf(f, "%010d %010d %010ld\n", reboot_flag, myth_reboots, myth_uptime);
	fprintf(f, "%d\n", sock);
	fcntl(sock, F_SETFD, 0);
	for(d = descriptor_list; d; d=d->next) {
		if(d) {
			process_output(d);
			if(d->state == CONNECTED && d->player && !d->internal) {
				if(reboot_flag < 2) {
					strcpy(buf, "");
					sprintf(buf, "%s|C+|[|Y+|REBOOT|C+|]|W+|: |C|Please hold on,|X| %s|C|, server is rebooting...\n",
						buf, color_name(d->player, -1));
					if(d->player->flags & ANSI)
						write(d->descriptor, colourize(buf), strlen(colourize(buf)));
					else
						write(d->descriptor, strip_color(buf), strlen(strip_color(buf)));
				}
				d->player->flags &= ~CONNECT;
				fprintf(f, "%010d %010d %010ld %010ld\n", d->descriptor, d->player->ref,
					d->connected_at, d->last_time);
				fcntl(d->descriptor, F_SETFD, 0);
			} else {
				if(reboot_flag < 2) {
					sprintf(buf, "|C+|[|Y+|REBOOT|C+|]|W+|: |C|Please try back after the reboot...\n");
					write(d->descriptor, colourize(buf), strlen(colourize(buf)));
				}
				close(d->descriptor);
			}
		}
	}
	
	fclose(f);
}

void set_sockets(int amt)
{
	int s;
	
	for(s = 0; s < amt; s++)
		fcntl(s, F_SETFD, 1);
}

void resort_desclist()
{
	DESCRIPTOR *d, *dtmp, *dnext;
	
	dtmp = descriptor_list;
	descriptor_list = NULL;
	
	for(d = dtmp; d; d=dnext) {
		dnext = d->next;
		d->next = descriptor_list;
		descriptor_list = d;
	}
}
		
void open_sockets()
{
	DESCRIPTOR *d;
	FILE *f;
	char buf[MAX_BUFSIZE];

#ifdef CATCHSEG	
	if( (f = fopen("panic_list", "r")) ) {
		unlink("panic_list");	// we must have segfaulted, restore the descriptor_list.
	} else {
#endif /*CATCHSEG*/
		if(!(f = fopen("socket_table", "r")))
			return;	// Clean boot up; refresh everything.
		
		unlink("socket_table");	// Proper reboot, load us up.
#ifdef CATCHSEG
	}
#endif /*CATCHSEG*/

	/* Grab the first bunch of data. */
	printf("%d size", sizeof(buf));
	fgets(buf, sizeof(buf), f);
	
	reboot_style = atoi(buf);	// get the first integer.
	myth_reboots = atoi(buf + 11) + 1;	// get number of reboots.
	myth_uptime = atoi(buf + 22);	// get the myth uptime.	
	myth_reboot = time(0);		// reset the myth reboot time.
	
	/* Grab the second bunch of data - namely the socket number */
	fgets(buf, sizeof(buf), f);
	
	sock = atoi(buf);		// Get this socket number.
	fcntl(sock, F_SETFD, 1);	// reset the socket flag.
	close(sock);			// close it up.
	
	set_sockets(1000);		// loop and set (X) number of sockets.

	/* Get the remaining data in its chunks and parse them. */
	while( fgets(buf, sizeof(buf), f) ) {
		int desc = atoi(buf);
		struct sockaddr_in in;
		int namelen = sizeof(in);
		
		fcntl(desc, F_SETFD, 1);
		getpeername(desc, (struct sockaddr *)&in, &namelen);
		
		{
			char buf[100];
			
			strcpy(buf, inet_ntoa(in.sin_addr));
			d = initializesock(desc, &in, buf, RELOADCONNECT);
		}
		
		// get the player reference.
		d->ref = atoi(buf+11);
		
		// get the connected at and last command time.
		d->connected_at = atoi(buf+22);
		d->last_time = atoi(buf+33);
		if(reboot_style == 4)
			wizlock = 1;
		else
			wizlock = 0;
	}				
	
	fclose(f);
	
	// let's flip the descriptor list back the way it should be!
	resort_desclist();
#ifdef CATCHSEG
	panic_list();
#endif CATCHSEG
}

struct descriptor_data *new_connection	(sock)
	int sock;
{
	int newsock;
	struct sockaddr_in addr;
	int addr_len = sizeof(addr);
	char buff[100];

	newsock = accept(sock, (struct sockaddr *)&addr, &addr_len);
	if (newsock < 0) {
		printf("returning with error\n");
		return NULL;
	} else {
		strcpy(buff, inet_ntoa(addr.sin_addr));
	}
	
	
	return initializesock(newsock, &addr, buff, WAITCONNECT);
}

void dwrite(d, buf) 
	DESCRIPTOR *d;
	char *buf;
{
	total_output += strlen(buf);
	
	if(!d) {
		printf("PANIC ERROR: Trying to write to NULL descriptor!\n");
		return;
	}
	d->output_queue = add_to_tqueue(d->output_queue, buf);
	d->output_lines++;
	d->output_flag = 1;
}

#ifdef CATCHSEG
/* Dump the panic list on connection */
void panic_list()
{
	DESCRIPTOR *d;
	FILE *f;
	
	f = fopen("panic_list", "w");
	
	reboot_flag = 3;
	fprintf(f, "%010d %010d %010ld\n", reboot_flag, myth_reboots, myth_uptime);
	reboot_flag = 0;
        fprintf(f, "%d\n", sock);
	for(d = descriptor_list; d; d=d->next)
		if(d->state == CONNECTED && d->player)
			fprintf(f, "%010d %010d %010ld %010ld\n", d->descriptor, d->player->ref, d->connected_at, d->last_time);

	fclose(f);
}
#endif /*CATCHSEG*/

struct descriptor_data *initializesock(s, a, addr, state)
	int s;
	struct sockaddr_in *a;
	char *addr;
	int state;
{
	struct descriptor_data *d;
	char buf[1024];
	
	GENERATE(d, DESCRIPTOR);
	d->descriptor = s;
	make_nonblocking(s);

	/** Set flags **/
	d->ref = 0;
	d->state = state;
	d->internal = SET_DONE;

	d->input_flag = 0;
	d->paste_flag = 0;
	d->output_flag = 0;
	d->output_lines = 0;
	d->paste_lines = 0;
	
	// text queues.
	d->input_queue = NULL;
	d->paste_queue = NULL;
	d->output_queue = NULL; 
	
	d->paste_cmd = NULL;

	// time values
	d->connected_at = time(&now);
	d->last_time = time(&now);
	
	// address stuff.
	d->address = *a;
	strncpy(d->addr, addr, 50);
	
	// figure out output on state
	switch(d->state) {
		case RELOADCONNECT:
			d->state = CONNECTED;
			break;		
		case WAITCONNECT:
			log_channel("log_io", pstr("|R+|* |W+|(|C+|ESTABLISHED LINK|W+|) [|G+|%d|W+|] |C|from |R+|%s |C|at %s",
				d->descriptor, d->addr, get_date(d->connected_at)));
			sprintf(buf, "|C+|[|B+|%s|C+|] |C|user|W+|@|G+|%s |W+|(|+R|Build|C+|: |Y+|%d|W+|)\n",
				return_config("myth_name"), addr, BUILD_NUM);
			dwrite(d, buf);
			show_welcome(d);
			dwrite(d, "Login: ");
			break;
		default:
			break;
	}	
	
	// 0 out the player.
	d->player = NULL;
	
	// add to the descriptor_list;
	d->next = descriptor_list;	
	descriptor_list = d;
	
	// add to descriptor records stuffies
	ndescriptors++;
	if(d->descriptor >= maxd)
		maxd = d->descriptor+1;
	
	return d;
}

void make_nonblocking(s)
	int s;
{
	if(fcntl(s, F_SETFL, FNDELAY) == -1) {
		printf("error in nonblocking!\n");
	}
}

TQUEUE *add_to_tqueue(tqueue, buffer) 
	TQUEUE *tqueue;
	char *buffer;
{
	TQUEUE *t, *tnew, *tprev = NULL;
	
	if(!*buffer)
		return (tqueue);
	
	if(tqueue) {
		for(t = tqueue; t; t=t->next)
			tprev = t;
	}
		
	GENERATE(tnew, TQUEUE);
	tnew->buffer = NULL;
	SET(&tnew->buffer, buffer);
	tnew->next = NULL;
		
	if(!tprev) 
		tqueue = tnew;
	else 
		tprev->next = tnew;
		
	return tqueue;
}

int process_input(d)
	DESCRIPTOR *d;
{
	char *p, *s, *buffer = NULL, *qend;
	char buf[MIN_BUFSIZE];
	int got;	
	
	if( (got = read(d->descriptor, buf, sizeof(buf))) == 0)
		return 0;
	else if (got == -1)
		return 1;
	
	strncpy(buf, buf, MIN_BUFSIZE);
	SET(&buffer, buf);
	s = buffer;
		
	for(p = buf, qend = buf + got;p < qend; p++) {
		if(*p == '\n') {
			*s = '\0';
			total_input += strlen(buffer);
			if (s >= buffer)
				d->input_queue = add_to_tqueue(d->input_queue, buffer);
			s = buffer;
		} else if (isascii(*p) && isprint(*p)) {
			*s++ = *p;
		}
	}
	
	if(buffer)
		free(buffer);
	d->last_time = time(0);
	d->input_flag = 1;
	return 1;
}

void connect_player(d)
	DESCRIPTOR *d;
{
	d->state = CONNECTED;
	d->player->flags |= CONNECT;
#ifdef CATCHSEG
	panic_list();
#endif /*CATCHSEG*/
	printf("we are here!\n");
	log_channel("connect", pstr("|R+|* |W+|(|C+|CONNECT|W+|) |X|%s", 
		color_name(d->player, -1)));
	log_channel("log_io", pstr("|R+|* |W+|(|C+|CONNECT|W+|) [|G+|%d|W+|] |X|%s |C|from |M|%s |C|on %s",
		d->descriptor, color_name(d->player, -1), d->addr, get_date(d->connected_at)));
	__announce_mail(d->player);

	filter_command(d->player, "look");
	/* Show whatever file you want, probably MOTD */
	display_file(d, "msgs/motd");
	printf("and here!\n");

	notify(d->player, pstr("|C|You have connected from...|W+|: |C+|%s",d->addr));
	notify_in(d->player->location, NULL, pstr("%s |C|has connected!", color_name(d->player, -1)));
	printf("we get passed here\n");
}

void parse_password(d, buffer)
	DESCRIPTOR *d;
	char *buffer;
{
	
	if(!dcrypt(d->player->pass, buffer)) {
		echo_on(d, "");
		connect_player(d);
	} else {
		dwrite(d, "[ERROR]: Invalid Password.\n");
		dwrite(d, "Password: ");
	}
}

void process_commands(void)
{
	DESCRIPTOR *d, *dnext = NULL;
	TQUEUE *t;
	int flagit = 0;
	
	for(d=descriptor_list; d; d=dnext) {
		dnext = d->next;
		if(d->input_flag) {
			for(t = d->input_queue; t; t=t->next) {
				if(!strcmp(t->buffer, "QUIT"))
					flagit = 1;					
				else if(d->state == CONNECTED)
					if (d->paste_flag)
						do_paste(d, t->buffer);
					else 
						do_command(d, t->buffer);
				else if (d->state == WAITCONNECT)
					parse_connect(d, t->buffer);
				else if (d->state == WAITPASS)
					parse_password(d, t->buffer);
			}
			
			d->input_queue = TQUEUE_FREE(d->input_queue);
			d->input_flag = 0;	
			if(flagit) {
				dfree(d->descriptor);
				flagit = 0;
			}
		}
	}
}

int create_guest(d)
	DESCRIPTOR *d;
{
	object *o = NULL;
	char guest[MAX_BUFSIZE];
	int ctr;
	
	for(ctr = 0; ctr < MAX_GUESTS; ctr++) {
		sprintf(guest, "%s%d", GUEST_PREFIX, ctr);
		if(!(o = match_player(guest)))
			break;
	}
	
	if(o)
		return 0;
	
	o = create_player(rootobj, guest, guest);
	o->powlvl = 25;
	o->flags |= GUEST;
	d->player = o;
	d->state = CONNECTED;	
	filter_command(o, "ch join guest");
	filter_command(o, "ch guest default");
	o->powlvl = 10;
	connect_player(d);
	return 1;
}

// Connect a player!
void parse_connect(d, instr)
	DESCRIPTOR *d;
	char *instr;
{
	OBJECT *o;
	char safestr[MAX_BUFSIZE];
	
	if(!*instr)
		return;

	strcpy(safestr, instr);
	
	if(!strcmpm(front(safestr), "guest")) {
		if(!wizlock) {
			if(!create_guest(d)) {
				dwrite(d, "|Y+|* |R+|Sorry, all guest IDs are busy...\n");
				dfree(d->descriptor);
				return;
			} 
		} else {
			dwrite(d, "You cannot log in!");
			dfree(d->descriptor);
			return;
		}
		return;
	}

	if(!(o = match_player(safestr))) {
		dwrite(d, "[ERROR]: Invalid login name.\n");
		dwrite(d, "Login: ");
		return;
	}
	if(!(o->flags & WIZARD) && wizlock) {
		dwrite(d, "|Y+|* |R+|Wizard lock in effect; please try back later.\n");
		dfree(d->descriptor);
		return;
	}
	echo_off(d, "Password: ");
	d->state = WAITPASS;
	d->player = o;

	return;
}

void do_command(d, instr)
	DESCRIPTOR *d;
	char *instr;
{
	filter_command(d->player, instr);
}

// delete an entire queue.
// we want to go through and clear every thing from the queue.
TQUEUE *TQUEUE_FREE(tqueue)
	TQUEUE *tqueue;
{
	TQUEUE *t, *tnext;
	
	for(t = tqueue; t; t=tnext) {	
		tnext = t->next;
		// clear the buffer
		if(t->buffer)
			free(t->buffer);
		free(t);
	}
	
	return (NULL);
}

// delete a descriptor.
void dfree(ddel)
	int ddel;
{
	DESCRIPTOR *d, *dprev=NULL;	
	int catchref=-1, prevstate = -1;
	
	for(d=descriptor_list; d; d=d->next) {
		if(d->descriptor == ddel)
			break;
		dprev = d;
	}
	
	if(!d)
		return;
	
	process_output(d);
	close(d->descriptor);
	
	if(!dprev)
		descriptor_list = d->next;
	else
		dprev->next = d->next;
	
	prevstate = d->state;
	// check to see if player is connected on another descriptor.
	if(d->player) {
		catchref = d->player->ref;
		d->player->flags &= ~CONNECT;
		
	}
	
	log_channel("log_io", pstr("|R+|* |W+|(|C+|DISCONNECT|W+|) [|G+|%d|W+|] |X|%s |C|disconnected on %s",
		d->descriptor, (d->player) ? color_name(d->player, -1) : "|R+|UNKNOWN", get_date(time(0))));
	
	if(d->player)
		log_channel("connect", pstr("|R+|* |X|%s |C|has disconnected.", color_name(d->player, -1)));
	// if this player is flagged GUEST, get rid of it.
	if(d->player) {
		if(d->player->flags & GUEST) {
			guest_precycle(rootobj, d->player);
			d->player = 0;
		}
	}
	
	if(d->paste_cmd)
		free(d->paste_cmd);
	
	d->paste_queue = TQUEUE_FREE(d->paste_queue);
			
	free(d);
	
	// check to see if the player is connected elsewhere.
	for(d=descriptor_list; d; d=d->next)
		if(d->state == CONNECTED)
			if(catchref == d->player->ref)
				d->player->flags |= CONNECT;
	
#ifdef CATCHSEG
	panic_list();
#endif /*CATCHSEG*/
}

static void sig_report(int t_sig)
{
	int i;
	
	for(i = 0; sig_table[i].retstr; i++)
		if(sig_table[i].t_sig == t_sig)
			printf("Exit due to signal: %s\n", sig_table[i].retstr);

	printf("Exiting due to signal: %d\n", t_sig);
	exit(t_sig);			
}

#ifdef CATCHSEG
void panic_boot()
{
	FILE *f;
	int scramble = 0;
	char buf[1024];
	char *argv[] = {"myth", NULL};
	
	printf("** PANIC **\n");
	printf("We have crashed! Forking a new process (25 second wait)\n");
	/* TODO:
		Write some code or a watchdog that will not have to use potentially
		fubared code to reboot.
		
		As for now, protect against this by not saving the descriptor_list.
		My idea would be to do an actual descriptor_list dump on every log
		in; and if that foobared it would just reboot and use that descriptor
		list. But if there was no problem, then on a real reboot it would
		use a 'socket_table' dump and delete the other descriptor list dump
		
		** And if it crashed dumping that descriptor_list, at least it would
		still reboot the server.
	*/
	
	/* at least try and keep the socket open */
	fcntl(sock, F_SETFD, 1);
	if( (f = fopen("panic_list", "r")) ) {
		scramble= getint(f);
		scramble = getint(f);
		while(fgets(buf, sizeof(buf), f))
			fcntl(atoi(buf), F_SETFD, 0);
	}
	fclose(f);

	sleep(10);
		
	if(fork()==0) {
		signal(SIGSEGV, SIG_DFL);
		kill(getpid(), 11);
		exit(0);
	}
	
	generate_stack();	
	execv("myth", argv);
	unlink("socket_table");	
}
#endif /*CATCHSEG*/

static void set_signals(void)
{
	signal(SIGPIPE, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);
	signal(SIGQUIT, sig_report);
	signal(SIGHUP, SIG_IGN);
	signal(SIGILL, sig_report);
#ifdef CATCHSEG
	signal(SIGSEGV, panic_boot);
	signal(SIGFPE, panic_boot);
#endif
	signal(SIGKILL, sig_report);
	signal(SIGSTOP, sig_report);
	signal(SIGTSTP, sig_report);
}

void redirect_io(void)
{
	int fp;
	
	close(fileno(stdin));
	
	fp = open("log.it", O_WRONLY | O_CREAT | O_APPEND, DEF_MODE);
	close(fileno(stdout));
	
	dup2(fp, fileno(stdout));
	setvbuf(stdout, NULL, _IOLBF, 0);
	
	close(fp);
}

void parse_cmd(char *argv[])
{
	if (*++argv) {
		if (!strcmp(*argv,"-newdb")) {
			printf("Creating new database...\n");
			new_database();
			// let's quit now.
			exit(0);
		} else if (!strcmp(*argv,"-config")) {
			printf("MYTH Configuration...\n");
			setup_myth();
			// let's quit now.
			exit(0);
		}
	}
}

void echo_off(DESCRIPTOR *d, char *string)
{
	char buf[MAX_BUFSIZE];
	char echo_off_str[]={IAC,WILL,TELOPT_ECHO,'\0'};
	
	sprintf(buf, "%s%s", string, echo_off_str);
	dwrite(d, buf);
}

void echo_on(DESCRIPTOR *d, char *string)
{
	char buf[MAX_BUFSIZE];
	char echo_on_str[]={IAC,WONT,TELOPT_ECHO,'\0'};
	
	sprintf(buf, "%s%s", string, echo_on_str);
	dwrite(d, buf);
}

void shutdownsock(player, arg1)
	dbref player;
	char *arg1;
{
	int descriptor;
	
	if ( (descriptor = atoi(arg1)) < 1)
		return;

		
}

void boot_off(player)
	dbref player;
{
	DESCRIPTOR *d, *dnext = NULL;
	
	for(d=descriptor_list; d; d=dnext) {	
		dnext = d->next;
		if(d->state == CONNECTED || d->state == WAITPASS ) {
			if(d->player->ref == player)
				dfree(d->descriptor);
		}
	}
}

	
// re-attach objects to decriptor_list[x]s.
void attach_players()
{
	DESCRIPTOR *d;
	object *o;

	if(!reboot_style)
		return;
			
	for(o=player_list; o; o=o->next)
		for(d = descriptor_list; d; d=d->next)
			if(d->ref == o->ref) {
				d->player = o;
				d->player->flags |= CONNECT;
#ifdef CATCHSEG
				panic_list();
#endif /*CATCHSEG*/
			}
}

void do_resock()
{
	sock = make_socket(atoi(return_config("port")));
}

void report_online()
{
	if(reboot_style == 0)
		return;
		
	if(reboot_style == 3) {
		reboot_style = 0;
		world_dwrite("|Y+|[|R+|PANIC ERROR|Y+|] |Y+|* |C+|Illegal memory reference - server just rebooted.\n");
	} else if (reboot_style > 0) {
		world_dwrite("|C+|[|Y+|REBOOT|C+|]|W+|: |C|Server restart completed.\n");
	}	
	reboot_style = 0;
	log_channel("log_io", pstr("|Y+|* |X|%s |C|resumes operation...",
		return_config("myth_name")));
}
