/****************************************************************************
 MYTH - ANSI System
 Coded by Saruk (02/07/99)
 ---
 This ANSI system (which is STILL not as beautiful as my wife) has been modeled
 on the DreamsMAZE, MAZE v2 ANSI system.
 ****************************************************************************/
#include <stdlib.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

// our color type structure. 
struct color_t {
	char ccode;
	int retcode;
};

// our table of colors.
struct color_t colors[]= {
	{'X',	0},		// normal
	{'+',   1},		// bright/bold
	{'u',   4},		// underscore
	{'b',   5},		// blink
	{'r',   7},		// reverse
	{'N',   30},		// Black
	{'R',   31},		// Red
	{'G',   32},		// Green
	{'Y',   33},		// Yellow
	{'B',   34},		// Blue
	{'M',   35},		// Magenta
	{'C',   36},		// Cyan
	{'W',   37},		// White
	{'0',   40}, 		// BlackB
	{'1',   41}, 		// RedB
	{'2',	42},		// GreenB
	{'3', 	43},		// YellowB
	{'4',   44},		// BlueB
	{'5',	45},		// MagentaB
	{'6',	46},		// CyanB
	{'7',	47},		// WhiteB
	{'\0',  -1},		// (NULL)
};


// return color code from table.
int get_color(ch)
	char ch;
{
	int i;
	
	for (i = 0; colors[i].ccode; i++)
		if(colors[i].ccode == ch)
			return (colors[i].retcode);
	
	return (-1);
}

// find out if string is a color string.
int is_color(char *str)
{
	char buffer[1024];
	char *tmpstr=NULL, *p=NULL;
	
	strcpy(buffer, str);
	tmpstr = buffer;
	
	if(!(p=strchr(tmpstr, '|')))
		return(0);
	
	if(p==tmpstr)
		return(0);
	
	*p='\0';
	for(p=tmpstr; *p; p++)
		if(get_color(*p)==-1)
			return(0);
	
	return 1;
}

// return the ANSI codes for color.
char *translate_color(str, add)
	char *str;
	int add;
{
	static char buffer[MAX_BUFSIZE];
	char newstr[MAX_BUFSIZE];
	char *p;
	int c;
	strcpy(newstr, str);
	
	if(add)
		strcpy(buffer, "\e[");
	else
		strcpy(buffer, "");
	
	for(p=newstr;*p;p++)
	{
		if( (c=get_color(*p)) != -1) {
			if(add) {
				if(!c)
					sprintf(buffer, "%s;37;%d", buffer, c);
				else
					sprintf(buffer, "%s;%d", buffer, c);
			}
		} else {
			sprintf(buffer, "%c%s%c", '|', str, '|');
			return (buffer);
		}
	}
	
	if(add)
		sprintf(buffer, "%sm", buffer);	
	
	return(buffer);			
}

// morph color code into ANSI color code.
char *addcolor(str, flag)
	char **str;
	int flag;
{
	char *p = parse_it(str, '|');
	return(translate_color(p, flag));
}

// 0 strips, 1 colours
char *color(str, flag, len)
	char *str;
	int flag;
	int len;
{
	char *newstr = NULL, *p = NULL, *k = NULL;
	static char buffer[1024];
	char buffer2[1024];
	
	strcpy(buffer, "");
	strcpy(buffer2, str);
	newstr = buffer2;
	k = newstr;
	
	// for the counting, when 
	for(p=strchr(newstr, '|'); p; p = strchr(p, '|')) {
		if(is_color(p+1)) {
			*p++ = '\0';
			sprintf(buffer, "%s%s%s", buffer, k, addcolor(&p, flag));
			k = p;
		} else {
			p++;
		}
	}
	
	sprintf(buffer, "%s%s", buffer, k);
	return(buffer);
}

// colour a string.
char *colourize(str)
	char *str;
{
	static char retstr[MAX_BUFSIZE];
	
	strcpy(retstr, color(str,1, -1));
	sprintf(retstr, "%s\e[;%dm", color(str, 1, -1), 0);
	
	return(retstr);
}

// strip a colour string.
char *strip_color(str)
	char *str;
{
	return stralloc(color(str, 0, -1));
}

char *addcode(str)
	char **str;
{
	char *p = parse_it(str, '|');
	return(p);
}

/** This will return a truncated color-coded string. 
	a.) Strip string and compare to non-stripped string.
	if equal, return truncated
	b.) If not equal, add color codes in non colored string until end of truncation.
**/
char *color_len(str, len)
	char *str;
	int len;
{
	char *p = NULL, *k = NULL, *newstr = NULL;
	char stripped[MAX_BUFSIZE];
	char buf[MAX_BUFSIZE], buf2[MAX_BUFSIZE];
		
	strcpy(buf, str);
	newstr = buf;
	k = newstr;
	strcpy(stripped, strip_color(str));
	
	if(strlen(buf) == strlen(stripped)) {
		buf[len] = '\0';
		return stralloc(buf);
	}
	
	if(len >= strlen(stripped))
		return stralloc(buf);
	else
		stripped[len] = '\0';	

	/* This is the tricky part */
	strcpy(buf2, "");
	for(p = strchr(newstr, '|'); p; p = strchr(p, '|')) {
		if(is_color(p+1)) {
			*p++ = '\0';
			
			sprintf(buf2, "%s%s|%s|", buf2, k, addcode(&p));
			if(strlen(strip_color(buf2)) > strlen(stripped)) {
				while(strlen(strip_color(buf2)) != strlen(stripped)) 
					buf2[strlen(buf2) - 1] = '\0';
				return stralloc(buf2);
			} else if (strlen(strip_color(buf2)) == strlen(stripped))
				return stralloc(buf2);
			k = p;
		} else {
			p++;
		}
	}
	
	sprintf(buf2, "%s%s", buf2, k);
	if(strlen(strip_color(buf2)) > strlen(stripped)) {
		while(strlen(strip_color(buf2)) != strlen(stripped))
			buf2[strlen(buf2) - 1]='\0';
	}
		
	return stralloc(buf2);
	
}
	
// chop off colour at a specific length.
char *truncate_color(str, len)
	char *str;
	int len;
{
	return color_len(str, len);
}

// pad the end of a string with spaces.
char *spc(num)
	int num;
{
	static char buf[64];
	char *b = buf;
	int count = 0;
	
	for(b = buf;  count <num; count++)
		*b++ = ' ';
	
	*b = '\0';	
	return(buf);
}

// color my name baby.
char *color_name(object *player, int length)
{
	char buf[MAX_BUFSIZE];	
	
	strcpy(buf, atr_get(player, "Rainbow"));
	
	if(!*buf) {
		strcpy(buf, player->name);
		buf[length]='\0';
		return stringalloc(buf);
	}
	
	if(strcmp(strip_color(buf), player->name)) {
		strcpy(buf, player->name);
		buf[length]='\0';
		return stringalloc(buf);
	}
	
	if(strlen(buf) == strlen(player->name)) {
		if(strlen(player->name) > length)
			buf[length] = '\0';
		else 
			strcat(buf, spc(length - strlen(buf)));
			
		return stringalloc(buf);	
	}
	
	if(length != -1) {
		if(strlen(strip_color(buf)) >= length) {
			strcpy(buf, truncate_color(buf, length));
		} else {
			strcat(buf, spc(length - strlen(strip_color(buf))));
		}
	} 
	
	strcat(buf, "|X|");	
	return stringalloc(buf);
}

char *format_color(char *string, int length)
{
	char buf[MAX_BUFSIZE];

	if(!*string)
		return "";
		
	strcpy(buf, string);
	if(strlen(strip_color(string)) >= length) {
		strcpy(buf, truncate_color(buf, length));
	} else {
		strcat(buf, spc(length - strlen(strip_color(buf))));
	}
	
	strcat(buf, "|X|");
	return stralloc(buf);
}