/****************************************************************************
 MYTH - Command structure, parsing, etc.
 Coded by Saruk (01/25/99)
 ---
 Command filtering, processing, etc.
 ****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>

#include "db.h"
#include "net.h"
#include "externs.h"
#include "cmdlist.h"

// command.c specifics
void do_alias P((object *, char *, char *));
void add_alias P((object *, char *, char *));
void del_alias P((object *, char *));

// command variables
int recursion = 0;

void do_listcmd(player)
	object *player;
{
	int index;
	
	notify(player, "|B+|Available |C+|Commands|W+|:");
	for(index = 0; cmdlist[index].command;index++)
		if((player->powlvl >= cmdlist[index].powerlevel) && ( ((cmdlist[index].flags & WIZARD) &&
			(player->flags & WIZARD)) || !(cmdlist[index].flags & WIZARD)) )
			notify(player, pstr("|C+|[|%s|%-15.15s|C+|] [|G+|%3d|W+|/|M+|%3d|C+|]",
				(cmdlist[index].flags & WIZARD) ? "R+":"Y+", 
				cmdlist[index].command, cmdlist[index].powerlevel, player->powlvl));
}

void do_reboot(player, style)
	object *player;
	char *style;
{
	DESCRIPTOR *d, *dnext = NULL;

	if(!strcmpm(style, "quiet")) {
		notify(player, "|C+|[|Y+|WIZARD|C+|]|W+|: |C|Initiating a quiet reboot...");
			
		log_channel("log_io", pstr("|R+|* |X|%s |C|initiated a |C+|quiet |C|reboot...",
			color_name(player, -1)));
		reboot_flag = 2;
	} else if (!strcmpm(style, "wizard")) {
		notify(player, "|C+|[|Y+|WIZARD|C+|]|W+|: |C|Initiating a wizard reboot...");
		log_channel("lo_io", pstr("|R+|* |X|%s |C|initiated a |C+|wizard|C| reboot...",
			color_name(player, -1)));
		for(d=descriptor_list;d;d=dnext) {
			dnext = d->next;
			if(d->state == CONNECTED && d->player)
				if(!(d->player->flags & WIZARD)) {
					dwrite(d, "|C+|[|Y+|REBOOT|C+|]|W+|: |C|Wizard reboot - sorry, disconnecting you...\n");
					boot_off(d->player->ref);
				}
		}
		reboot_flag = 4;
	} else {
		notify(player, "|C+|[|Y+|WIZARD|C+|]|W+|: |C|Initiating a reboot...");
		log_channel("log_io", pstr("|R+|* |X|%s |C|initiated a |C+|standard|C| reboot...",
			color_name(player, -1)));
		reboot_flag = 1;
	}
		
	shutdown_flag = 1;
}

void do_shutdown(player)
	object *player;
{
	DESCRIPTOR *d;
	
	for(d=descriptor_list;d;d=d->next)
		if(d->state == CONNECTED)
			dwrite(d, pstr("|C+|[|Y+|SHUTDOWN|C+|]|W+|:|X| %s |R+|caused a shutdown!\n", 
				color_name(player, -1)));
	shutdown_flag = 1;
}

int check_ambiguous(player, cmd)
	object *player;
	char *cmd;
{
	char buf[MAX_BUFSIZE];
	int ctr, amt;
	
	strcpy(buf, "");
	for(ctr = 0, amt = 0; cmdlist[ctr].command; ctr++)
		if(string_prefix(cmd, cmdlist[ctr].command)) {
			if( (cmdlist[ctr].flags & NOAMB) || !strcmpm(cmdlist[ctr].command, cmd) )
				return 0;
			amt++;
			sprintf(buf, "%s%s%s", buf, (*buf) ? " ":"", cmdlist[ctr].command);
		}
		
	if(amt>1) {
		notify(player, pstr("Ambiguous Commands: %s", buf));
		return 1;
	}
	
	return 0;
}

void filter_command(player, instr)
	object *player;
	char *instr;
{
	CLIST *clist;
	struct cmd_list_t *fp;
	char a[1024], b[1024], c[1024], pure[1024];	
	int index = 0;
	
	if(player->flags & SUSPECT)
		log_channel("log_suspect", pstr("|R+|* |W+|(|C+|%s|W+|) |X|%s|Y+|->|W+|: |X|%s", 
			get_date(time(0)), color_name(player, -1), instr));
	
	if(player->flags & GUEST)
		log_channel("log_guest", pstr("|R+|* |W+|(|C+|%s|W+|) |X|%s|Y+|->|W+|: |X|%s",
                        get_date(time(0)), color_name(player, -1), instr));	
        
        while(*instr && isspace(*instr))
		instr++;
	
	if(!*instr)
		return;
	
	strcpy(pure, restof(instr));
	
	if(player->powlvl > 5) {
		if(*instr == SAY_TOKEN) {
			instr++;
			DO_SAY(player, instr);
			return;
		} else if (*instr == POSE_TOKEN) {
			instr++;
			DO_POSE(player, instr);	
			return;	
		} else if (*instr == POSS_TOKEN) {
			instr++;
			DO_POSS(player, instr);
			return;
		} else if (*instr == THINK_TOKEN) {	
			DO_THINK(player, instr + 1);
			return;
		} else if (*instr == TO_TOKEN) {
			DO_TO(player, instr + 1);
			return;
		} else if (*instr == CHAN_TOKEN) {	
			do_channel_default(player, instr + 1);
			return;
		}
	}

	strcpy(a, front(instr));	
	

	/* MATCHING EXITS */
	// Pseudo Exits
	if( (index = match_pseudo_exit(player, a)) == 1)
		return;
	
	// Object Exits
	if(match_real_exit(player, a))
		return;
	
	if(index < 0) {
		notify(player, "|Y+|* |R+|You can't go that way...");
		return;
	}
	
	
		
	if(match_channels(player, a, restof(instr)))
		return;

	for(clist = player->cmdlist; clist; clist=clist->next)
		if( !strcmpm(clist->cmd, a) ) {
			if((recursion++) > (15)) {
				error_report(player, M_ERECURSE);
				recursion = 0;
				return;
			}

			strcpy(c, clist->send);
			if(strlen(restof(instr)) > 0)
				sprintf(c, "%s %s", c, restof(instr));
			filter_command(player, c);
			strcpy(c, "");
			return;
		}

	if(check_ambiguous(player, a))
		return;
		
	for(index = 0; cmdlist[index].command; index++) {
		fp = &cmdlist[index];
		if( string_prefix(a, fp->command) && (fp->powerlevel <= player->powlvl) &&
			( ((fp->flags & WIZARD) && (player->flags & WIZARD)) || !(fp->flags & WIZARD)) ) {
			if(fp->flags & TEXT) {
				strcpy(a, fp->command);
				strcpy(b, restof(instr));
			} else {
				strcpy(c, restof(instr));
				strcpy(a, front(c));
				strcpy(b, restof(c));
			}
			
			if(fp->flags & PURITY) 
				fp->func(player, pure);
			else if (fp->flags & SPEAK_SAY) {
				DO_SAY(player, pure);
			} else if (fp->flags & SPEAK_POSE) {
				DO_POSE(player, pure);
			} else if (fp->flags & SPEAK_THINK) {
				DO_THINK(player, pure);
			} else
				fp->func(player, a, b);
			
			recursion = 0;
			return;
		}
	}	
	
	
	error_report(player, M_EUNKCMD);	
}

// seen as how command aliasing is a command function...
void do_alias(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	CLIST *tmp;
	int ctr = 0;
	
	if(!*arg1 || !*arg2) {
		error_report(player, M_EINVARG);
		return;
	}
	
	if(!strcmpm(arg1, "delete")) {
		for(tmp = player->cmdlist;tmp; tmp=tmp->next)
			if(!strcmpm(arg2,tmp->cmd)) {
				notify(player, pstr("|C+|[|R+|DELETE|C+|]|W+|:|X| %s |C|was deleted.",
					tmp->cmd));
				del_alias(player, tmp->cmd);
				return;
			}
		
		error_report(player, M_EILLSTRING);
		return;
	}	
	
	for(ctr = 0; cmdlist[ctr].command; ctr++) {
		if(string_prefix(arg1, cmdlist[ctr].command)) {
			error_report(player, M_ESHDBLT);
			return;
		} 
	}
	
	add_alias(player, arg1, arg2);
}

void add_alias(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	CLIST *tmp;
	
	for(tmp = player->cmdlist; tmp; tmp=tmp->next)
		if(!strcmpm(tmp->cmd, arg1)) {
			error_report(player, M_ESHDBLT);
			return;
		}
	

	GENERATE(tmp, CLIST);
	tmp->cmd = NULL;
	tmp->send = NULL;
	SET(&tmp->cmd, arg1);
	SET(&tmp->send, arg2);

	tmp->next = player->cmdlist;
	player->cmdlist = tmp;
}

void del_alias(player, todel)
	object *player;
	char *todel;
{
	CLIST *c, *cprev = NULL;
	
	for(c = player->cmdlist; c; c=c->next) {
		if(!strcmpm(c->cmd, todel))
			break;
		cprev = c;
	}
	
	if(!c)
		return;
	
	if(!cprev)
		player->cmdlist = c->next;
	else
		cprev->next = c->next;
	
	if(c->cmd)
		free(c->cmd);
	if(c->send)
		free(c->send);
	free(c);
}

void do_alias_list(player, arg1)
	object *player;
	char *arg1;
{
	object *who;
	CLIST *c;
	
	if(*arg1) {
		if(!(who=match_player(arg1))) {
			error_report(player, M_ENOPLAY);
			return;
		}
	} else
		who = player;

	if(!can_control(player, who)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	notify(player, pstr("%s|C|'s |B+|Commands|W+|:", color_name(who, -1)));
	
	for(c=who->cmdlist;c;c=c->next)
		notify(player, pstr("|W+|[|R+|%-12.12s|W+|] |C|represents|W+|: |M+|%s",
			c->cmd, c->send));
		
}

CLIST *mkcmd(cmdlist, tmpstr)
	CLIST *cmdlist;
	char *tmpstr;
{
	CLIST *c;
	char buf[2048];
	
	strcpy(buf, tmpstr);
	GENERATE(c, CLIST);
	c->cmd = NULL;
	c->send = NULL;
	SET(&c->cmd, front(buf));
	SET(&c->send, restof(buf));

	c->next = cmdlist;
	cmdlist = c;
	return(cmdlist);
}