/****************************************************************************
 MYTH - Help System
 Coded by Saruk (02/06/99)
 ---
 The goal is to use this system like man pages. In fact, possibly using the 
 syntax as man pages.
 ****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

void do_version(player)
	object *player;
{
	make_header(player, "|W+|--- |C|Version Information |W+|---");
	notify(player, pstr("\t|B+|Code Based Off........|W+|: |Y+|%s\n", SERVER_BASE));
	notify(player, pstr("\t|B+|Current Code Version..|W+|: |Y+|%s |M+|(|C|Build #|Y+|%d|M+|)", SERVER_CODE, BUILD_NUM));
	notify(player, pstr("\t|B+|Database Version......|W+|: |Y+|MYTH DB v%d", DB_VERSION));
	notify(player, pstr("\t|B+|Main Developer........|W+|: |Y+|%s\n", DEVELOPER));
	notify(player, pstr("\t|B+|Previous Developers...|W+|: |Y+|%s", PREV_DEVELOPERS));
	notify(player, pstr("|B|%s", make_ln("-", 78)));
	notify(player, pstr("\t|B+|Compiler Options......|W+|: |Y+|%s %s",
#ifdef MALLOCREPORT
	"MALLOCREPORT",
#else
	"",
#endif
#ifdef CATCHSEG
	"CATCHSEG"
#else
	""
#endif
	));
	notify(player, pstr("\t|B+|Last Code Upgrade.....|W+|: |Y+|%s", get_date(LAST_BUILD_DATE)));
	notify(player, pstr("\t|B+|Copyright Date........|W+|: |Y+|%s", COPYRIGHT_DATE));
	notify(player, pstr("|B+|%s", make_ln("=", 78)));
}

void do_uptime(player)
	object *player;
{
	float throughput = 0, throughget = 0;
	time_t next_dump_time = 0;
	
	// Calculate throughput.	
	throughput = (float)total_input / (float)(time(0)-myth_reboot);
	throughget = (float)total_output / (float)(time(0)-myth_reboot);
	
	// Calculate next database dump.
	next_dump_time = last_dump_time + dump_time;

	make_header(player, "|W+|--- |C|Uptime Information |W+|---");
	notify(player, pstr("\t|B+|Initial Startup.......|W+|: |Y+|%s\n", get_date(myth_uptime)));
	if(myth_reboots) {
		notify(player, pstr("\t|B+|Last Reboot At........|W+|: |Y+|%s", get_date(myth_reboot)));
		notify(player, pstr("\t|B+|Total Reboots.........|W+|: |Y+|%d", myth_reboots));
	}
	notify(player, pstr("\t|B+|Current Time..........|W+|: |Y+|%s", get_date(time(0))));
	notify(player, pstr("\t|B+|Last Database Save....|W+|: |Y+|%s", get_date(last_dump_time)));
	notify(player, pstr("\t|B+|Next Database Number..|W+|: |Y+|%d", database_saves+1));
	notify(player, pstr("\t|B+|Next Database Save....|W+|: |Y+|%s\n", get_date(next_dump_time)));
	notify(player, pstr("\t|B+|Total Input/Second....|W+|: |Y+|%s |C|bytes |W+|(|G+|%.2f|M+|/sec|W+|)",
		comma((int)total_input), throughput));
	notify(player, pstr("\t|B+|Total Output/Second...|W+|: |Y+|%s |C|bytes |W+|(|G+|%.2f|M+|/sec|W+|)",
		comma((int)total_output), throughget));
	notify(player, pstr("|B|%s", make_ln("-", 78)));
	notify(player, pstr("|G+|%s",cjust("Elapsed Time", 78)));
	notify(player, pstr("|Y+|%s", cjust(time_format2(time(0) - myth_uptime),78)));
	notify(player, pstr("|M+|%s", cjust("Next Dump In", 78)));
	notify(player, pstr("|Y+|%s", cjust(time_format2(next_dump_time - time(0)), 78)));
	notify(player, pstr("|B+|%s", make_ln("=", 78)));
}

void do_help(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	FILE *fp;
	int show_lines = FALSE;		// whether or not to show lines. (default is false)
	int found_topic = FALSE;	// Error report on FALSE.
	char filename[MAX_BUFSIZE];	// filename to get.
	char buf[MAX_BUFSIZE];		// store buffer.
	char lookfor[MAX_BUFSIZE];	// what we're looking for.
		
	sprintf(filename, "%s/%s.txt", return_config("text_path"), arg1);
	
	if(!(fp = fopen(filename, "r"))) {
		error_report(player, M_ENOFILE);
		return;
	}

	strcpy(lookfor, (!*arg2) ? arg1 : arg2);
		
	while(fgets(buf,sizeof(buf), fp)) {
		if(buf[strlen(buf)-1]=='\n')
			buf[strlen(buf)-1] = '\0';
			
		if(buf[0] == '&' && found_topic == FALSE) {
			if(string_prefix(lookfor, buf + 1)) {
				show_lines = TRUE;
				notify(player, pstr("|Y+|TOPIC|W+|: |B+|%s", buf + 1));
			} else
				show_lines = FALSE;
			strcpy(buf, "");
		} else if (buf[0] == '&' && found_topic == TRUE) {
			show_lines = FALSE;
		}

		if(show_lines) {
			found_topic = TRUE;
			if(buf[0] == '.') 
				notify(player, pstr("|C+|See Also|W+|: |B+|%s", buf + 1));
			else
				notify(player, buf);
		}

	}

	if(!found_topic) 
		error_report(player, M_ENOTFOUND);

	fclose(fp);
}