/****************************************************************************
 MYTH - Report System
 Coded by Saruk (02/12/99)
 ---
 This is a report system for displaying data on all different sections of the
 MYTH server.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <netdb.h>

#include "db.h"
#include "net.h"
#include "externs.h"

void do_more_report(player, arg1)
	object *player;
	char *arg1;
{
	DESCRIPTOR *d;
	
	if(!(player->flags & WIZARD)) {
		error_report(player, M_ENOCANDO);
		return;
	}
	
	make_header(player, "|W+|--- |C|IO Report |W+|---");
	
	for(d = descriptor_list; d; d=d->next) {
		notify(player, pstr("|Y+|%d|W+|[|B+|%d|W+|] |W+|- |X|%s |C|from |C+|%s", d->descriptor, d->state, 
			(d->player) ? color_name(d->player, -1) : "|R+|UNKNOWN|X|", d->addr));
	}
}

void do_report(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	int report_stack_calls, report_stack_frees;
#ifdef MALLOCREPORT
	struct mallinfo M_MEMINF;
#endif

	if(!strcmpm(arg1, "io")) {	
		do_more_report(player, arg1);
		return;
	}
	
	make_header(player, "|W+|--- |C|Report System |W+|---");
	notify(player, pstr("\t|B+|Process ID.............|W+|: |Y+|%d", getpid()));
	notify(player, pstr("\t|B+|Root object............|W+|: |Y+|%s\n", color_name(rootobj, -1)));

	
#ifdef MALLOCREPORT	
	M_MEMINF = mallinfo();
	notify(player, pstr("\t|B+|Total Allocated Memory.|W+|: |Y+|%s |C|bytes.", comma(M_MEMINF.arena)));
	notify(player, pstr("\t|B+|Free Allocated Memory..|W+|: |Y+|%s |C|bytes.", comma(M_MEMINF.fordblks)));
	notify(player, pstr("\t|B+|Free Chunks of Memory..|W+|: |Y+|%d", M_MEMINF.ordblks));
	notify(player, pstr("\t|B+|Total Used Memory......|W+|: |Y+|%s |C|bytes.", comma(M_MEMINF.uordblks)));
	notify(player, pstr("\t|B+|Total Memory Activity..|W+|: |Y+|%s |C|bytes.", comma((int)tot_mem_act)));
	notify(player, pstr("\t|B+|Total Number of Mallocs|W+|: |Y+|%d", num_mallocs));
	notify(player, pstr("\t|B+|Total Number of Frees..|W+|: |Y+|%d", num_frees));
	notify(player, pstr("\t|B+|Number of Non-freed....|W+|: |Y+|%d", (num_mallocs - num_frees)));
#endif /*MALLOCREPORT*/
	clear_stack();
	report_stack_calls = total_stack_calls;	
	report_stack_frees = total_stack_frees;
	notify(player, pstr("\t|B+|Total Stack Calls......|W+|: |Y+|%d", report_stack_calls));
	notify(player, pstr("\t|B+|Total Stack Frees......|W+|: |Y+|%d", report_stack_frees));
	notify(player, pstr("|B+|%s", make_ln("=", 78)));
}

void do_dbinfo(player)
	object *player;
{
	object *o;
	attr *a;
	int num_rooms, num_players, num_guests, num_things, num_exits, total_num, player_owned, pseudo_exits;
	int num_wizards;
	
	pseudo_exits = num_rooms = num_players = num_guests = num_things = total_num = num_exits = 0;
	player_owned = num_wizards = 0;
	for(o = room_list; o; o=o->next) {
		if(o->ownit == player) player_owned++;
		num_rooms++;
		for(a = o->alist; a; a = a->next) {
			if((a->flags & AEXIT) && *(atr_get(o, a->name)))
				pseudo_exits++;
		}
	}
	
	for(o = player_list; o; o=o->next) {
		if(o->ownit == player) player_owned++;
		if(o->flags & GUEST)
			num_guests++;
		else
			num_players++;
		if(o->flags & WIZARD)
			num_wizards++;
	}
	for(o = exit_list; o; o=o->next) {
		if(o->ownit == player) player_owned++;
		num_exits++;
	}
	for(o = thing_list; o; o=o->next) {
		if(o->ownit == player) player_owned++;		
		num_things++;
	}
		
	total_num = num_players + num_rooms + num_guests + num_things + num_exits;
	
	make_header(player, "|W+|--- |C|Database Information |W+|---");
	notify(player, pstr("\t|B+|Objects owned by you...|W+|: |Y+|%d", player_owned));
	notify(player, pstr("\t|B+|Number of objects......|W+|: |Y+|%d", total_num));
	notify(player, pstr("\t|B+|Number of |R+|Wizards|B+|......|W+|: |Y+|%d\n", num_wizards));
	notify(player, pstr("\t|B+|Number of rooms........|W+|: |Y+|%d", num_rooms));
	notify(player, pstr("\t|B+|Number of exits........|W+|: |Y+|%d", num_exits));
	notify(player, pstr("\t|B+|Number of things.......|W+|: |Y+|%d", num_things));
	notify(player, pstr("\t|B+|Number of guests.......|W+|: |Y+|%d", num_guests));
	notify(player, pstr("\t|B+|Number of players......|W+|: |Y+|%d", num_players));
	notify(player, pstr("\t|B+|Number of pseudo exits.|W+|: |Y+|%d", pseudo_exits));
	notify(player, pstr("|B+|%s", make_ln("=", 78)));
}

void do_search(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	char buf[MAX_BUFSIZE], type[MAX_BUFSIZE];
	OBJECT *victim, *o;
	int style = 0, tell = 0;	// check style
	
	if (!*arg1)
		victim = player;
	else if(!strcmpm(arg1, "all")) {
		style = 1;
		victim = player;
	} else if (!strcmpm(arg1, "type")) {
		style = 2;
		strcpy(type, arg2);
		victim = player;
	} else {
		if(!(victim = match_player(arg1))) {
			error_report(player, M_ENOPLAY);
			return;
		}
		if(!can_control(player, victim)) {
			error_report(player, M_ENOCNTRL);
			return;
		}
		if(!strcmpm(front(arg2), "type")) {
			style = 2;
			strcpy(type, restof(arg2));
		}
	}
	
	strcpy(buf, color_name(victim, -1));
	notify(player, pstr("|C+|Searching|X| %s|C+|...", buf));
	
	/* PLAYERS SEARCH */
	for(o = player_list; o; o=o->next) {
		if(!style) { // obviously want to search self.
			if(o->ownit == victim)
				tell = 1;
		} else if (style == 1) {
			if(o->ownit == victim || can_control(victim, o->ownit))
				tell = 1;
		} else if (style == 2) {
			if(match_type(o, type))
				if(o->ownit == victim || can_control(victim, o->ownit))
					tell = 1;
		}
		if(tell) {
			strcpy(buf, color_name(o->ownit, -1));
			notify(player, pstr("|W+|(|C+|PLAYER|W+|)|X| %s |W+|(|Y+|%d%s|W+|) |C|is owned by|X| %s",
				color_name(o, -1), o->ref, flaglist(o), buf));
			tell = 0;
		}
	}
		


	/* THING SEARCH */
	for(o = thing_list; o; o=o->next) {
		if(!style) { // obviously want to search self.
			if(o->ownit == victim)
				tell = 1;
		} else if (style == 1) {
			if(o->ownit == victim || can_control(victim, o->ownit))
				tell = 1;
		} else if (style == 2) {
			if(match_type(o, type))
				if(o->ownit == victim || can_control(victim, o->ownit))
					tell = 1;
		}
		if(tell) {
			strcpy(buf, color_name(o->ownit, -1));
			notify(player, pstr("|W+|(|C+|THING|W+|)|X| %s |W+|(|Y+|%d%s|W+|) |C|is owned by|X| %s",
				color_name(o, -1), o->ref, flaglist(o), buf));
			tell = 0;
		}
	}
	
	/* ROOM SEARCH */
	for(o = room_list; o; o=o->next) {
		if(!style) { // obviously want to search self.
			if(o->ownit == victim)
				tell = 1;
		} else if (style == 1) {
			if(o->ownit == victim || can_control(victim, o->ownit))
				tell = 1;
		} else if (style == 2) {
			if(match_type(o, type))
				if(o->ownit == victim || can_control(victim, o->ownit))
					tell = 1;
		}
		if(tell) {
			strcpy(buf, color_name(o->ownit, -1));
			notify(player, pstr("|W+|(|C+|ROOM|W+|)|X| %s |W+|(|Y+|%d%s|W+|) |C|is owned by|X| %s",
				color_name(o, -1), o->ref, flaglist(o), buf));
			tell = 0;
		}
	}
		
	/* EXIT SEARCH */
	for(o = exit_list; o; o=o->next) {
		if(!style) { // obviously want to search self.
			if(o->ownit == victim)
				tell = 1;
		} else if (style == 1) {
			if(o->ownit == victim || can_control(victim, o->ownit))
				tell = 1;
		} else if (style == 2) {
			if(match_type(o, type))
				if(o->ownit == victim || can_control(victim, o->ownit))
					tell = 1;
		}
		if(tell) {
			strcpy(buf, color_name(o->ownit, -1));
			notify(player, pstr("|W+|(|C+|EXIT|W+|)|X| %s |W+|(|Y+|%d%s|W+|) |C|is owned by|X| %s",
				color_name(o, -1), o->ref, flaglist(o), buf));
			tell = 0;
		}
	}
		
}

