/****************************************************************************
 MYTH - (Bulletin Board System)
 Coded by Saruk (01/14/00)
 ---
 All the nifty BB functions.
 ****************************************************************************/
 
#include <stdio.h>
#include <stdlib.h> 
#include "db.h"
#include "net.h"
#include "externs.h"

BBOARD *add_to_board(player, title, comment)
	object *player;
	char *title;
	char *comment;
{
	BBOARD *b, *bnew, *bprev = NULL;
	
	for(b = bblist; b; b=b->next)
		bprev = b;
	
	GENERATE(bnew, BBOARD);
	bnew->topic = NULL;
	if(!*title)
		SET(&bnew->topic, "Untitled");
	else
		SET(&bnew->topic, title);
	bnew->message = NULL;
	SET(&bnew->message, comment);
	bnew->c_ref = player->ref;
	bnew->creator = player;
	bnew->created = time(0);
	bnew->expiration = time(0) + (DEF_BBEXP * 86400);
	bnew->num_read = 0;
	bnew->next = NULL;
	
	if(!bprev) 
		bblist = bnew;
	else
		bprev->next = bnew;
		
	return bnew;
}

void do_add_msg(player, msg)
	object *player;
	char *msg;
{
	int amt=0;
	BBOARD *b;
	char *title;
	char *message;
	
	for(b = bblist; b; b=b->next)
		if(b->creator == player)
			amt++;
	
	if(amt >= BOARD_MAX && !(player->flags & WIZARD)) {
		error_report(player, M_ETOOBOARD);
		return;
	}
			
	title = fparse(msg, '/');
	message = rparse(msg, '/');

	if(!*message)
		add_to_board(player, "", title);
	else 
		add_to_board(player, title, message);
	
	notify(player, "|C+|* |C|Your message has been added.");
	world_dwrite("|C+|* |C|A message has been added to the bulletin board!\n");
}

void do_del_msg(player, arg1)
	object *player;
	char *arg1;
{
	BBOARD *b, *bprev = NULL;
	int cnt, del;
	
	if( (del = match_num(arg1)) < 1) {
		error_report(player, M_ENOTFOUND);
		return;
	}
	
	for(b = bblist, cnt = 1; b; b=b->next, cnt++) {
		if(cnt == del)
			break;
		bprev = b;
	}
	
	if(!b) {
		error_report(player, M_ENOTFOUND);
		return;
	}
	
	if(!can_control(player, b->creator)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	if(!bprev)
		bblist = b->next;
	else
		bprev->next = b->next;
	
	free(b->topic);
	free(b->message);
	free(b);
	
	notify(player, pstr("|Y+|* |C+|%d |C|has been deleted.", del));
}


void do_board_view(player)
	object *player;
{
	BBOARD *b;
	int cnt = 1;
	
	notify(player, pstr("|B+|/|B|%s|B+|\\",make_ln("=",76)));
	notify(player, pstr("|B+|| |W+|%-3.3s|B+| | |W+|%-4.4s |B+|| |W+|%-4.4s |B+|| |W+|%-23.23s |B+|| |W+|%-28.28s |B+||", "NUM", 
		"READ", "AGE", "AUTHOR", "TITLE"));
	notify(player, pstr("|B+|||B|%s|B+|||B|%s|B+|||B|%s|B+|||B|%s|B+|||B|%s|B+||", make_ln("-",5), make_ln("-", 6), 
		make_ln("-", 6), make_ln("-",  25), make_ln("-", 30)));
	if(!bblist) 
		notify(player, pstr("|B+||%s|B+||", cjust("|Y+|Nothing to view!",76)));
	else {
		for(b = bblist; b; b=b->next) {
			notify(player, pstr("|B+|| |Y+|%3d |B+|| |G+|%4d |B+|| |R+|%-4.4s |B+|| |W|%s |B+|| %s |B+||", cnt, 
				b->num_read, count_idle(time(0)-b->created), color_name(b->creator, 23), format_color(b->topic, 28)));
			cnt++;
		}
			
	}
	notify(player, pstr("|B+|\\|B|%s|B+|/",make_ln("=",76)));
}

void do_read_board(player, arg1)
	object *player;
	char *arg1;
{
	int num, cnt;
	BBOARD *b;
	
	num = match_num(arg1);
	if(num < 1) {
		error_report(player, M_ENOTFOUND);
		return;
	}
	
	for(b=bblist,cnt=1;b;b=b->next,cnt++) 
		if(cnt == num) {
			notify(player, pstr("|B+|Title|W+|: |W|%s", b->topic));
			notify(player, pstr("|C+|Creator|W+|:|W| %s", color_name(b->creator, -1)));
			notify(player, pstr("|C+|Age|W+|:|Y+| %s |C+|Will Expire|W+|: |R+|%s", time_format2(time(0) - b->created),
				get_date(b->expiration)));
			notify(player, pstr("|C+|Number Read|W+|: |G+|%d", b->num_read));
			notify(player, pstr("|C+|Message|W+|: |W|%s", b->message));
			b->num_read++;
			return;
		}
	
	error_report(player, M_ENOTFOUND);
		
}

void do_board(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	if(!*arg1) {
		do_board_view(player);
		return;
	}

	if(!strcmpm(arg1, "add")) {
		if(!*arg2) {
			error_report(player, M_EINVARG);
			return;
		}
		do_add_msg(player, arg2);	
	} else if (!strcmpm(arg1, "del")) {
		if(!*arg2) {
			error_report(player, M_EINVARG);
			return;
		}
		do_del_msg(player, arg2);
	} else {
		do_read_board(player, arg1);
	}
}

void sort_bboard()
{
	BBOARD *b;
	int cnt;
	
	for(b = bblist, cnt = 1; b; b=b->next, cnt++)
		if(b->expiration <= time(0)) {
			log_channel("database", pstr("|R+|* |C|Bulletin board message |C+|%d|C| has expired: |R+|DELETED|C|.",
				cnt));
			do_del_msg(rootobj, pstr("%d", cnt));
		}
}