/****************************************************************************
 MYTH - Player communications handling.
 Coded by Saruk (01/23/99)
 ---
 Takes care of player creation and other player-specific functions.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

// substitute percent r (%r)
char *substitute(char *in)
{
	static char buf[MAX_BUFSIZE];
	char *p=buf;
	char *s;
	
	for(s = in; *s; s++, p++)
		if(*s == '%') {
			s++;
			if(*s == 'r') {
				*p = '\n';
			} else if (*s == 't') {
				*p = '\t';
			} else if (*s == 'b') {
				*p = ' ';
			} else if (*s =='a') {
				*p = '\a';
			} else {
				*p = *s;  
			}
		} else {
			*p = *s;
		}
	*p++ = '\0';
	return buf;
}

// lowest level notify player procedure.
void __notify(player, sendstr, tag)
	OBJECT *player;
	char *sendstr;
	int tag;
{	
	DESCRIPTOR *d;
	
	for (d=descriptor_list; d; d=d->next)
		if(d->state == CONNECTED) {
			if( (d->player->ref == player->ref)) {
				dwrite(d, sendstr);
				if(tag)
					dwrite(d, "\n");
			}
		}
}

// our usual notify procedure.
void notify(player, sendstr)
	OBJECT *player;
	char *sendstr;
{
	__notify(player, sendstr, 1);
}

void clear_exception(exception)
	CONTENTS *exception;
{
	CONTENTS *c, *cnext = NULL;
	
	if(!exception) {
		exception = NULL;
		return;
	}
		
	for(c = exception; c; c=cnext) {
		cnext = c->next;
		free(c);
	}

	if(exception)
		exception = NULL;
}

void add_exception(exception, player)
	CONTENTS **exception;
	OBJECT *player;
{
	CONTENTS *newc;
	
	if(!*exception)
		*exception = NULL;
		
	GENERATE(newc, CONTENTS);
	newc->content = player;
	newc->next = *exception;
	(*exception) = newc;
}

// notify in (aside from exception) object.
void notify_in(location, exception, sendstr)
	object *location;
	CONTENTS *exception;
	char *sendstr;
{
	CONTENTS *c, *e = NULL;
	int flagged = 0;
		
	for(c = location->contents; c; c=c->next) {
		flagged = 0;
		for (e = exception; e; e=e->next)
			if(e->content == c->content)
				flagged = 1;
		if( (c->content->flags & TYPE_PLAYER) && (c->content->flags & CONNECT) && !flagged)
			__notify(c->content, sendstr, 1);
	}
}

// MOVE THIS TO: misc/error.c
void perm_denied(player, str)
	OBJECT *player;
	char *str;
{
	notify(player, pstr("|B+|[|Y+|ERROR|B+|]|W+|:|X| %s", str));
}

// Parse speech
char *speech_parse(player, saystr)
	object *player;
	char *saystr;
{
	static char buf[1024];
	
	if(*saystr == ':')
		strcpy(buf, pstr("|X|%s |X|%s", color_name(player, -1), substitute(saystr+1)));
	else if (*saystr == ';')
		strcpy(buf, pstr("|X|%s|X|'s %s", color_name(player, -1), substitute(saystr+1)));
	else if (*saystr == '.') {
		strcpy(buf, pstr("|X|%s |N+|. |B|o |B+|O |C+|(|X| %s |C+|)",
			color_name(player, -1), substitute(saystr+1)));
	} else 
		strcpy(buf, substitute(saystr));
	
	return buf;
}

char *spoof(player, o)
	object *player;
	object *o;
{
	static char buf[MAX_BUFSIZE];
	
	if(o->flags & NOSPOOF)
		sprintf(buf, " |W+|(|Y+|%d|W+|)", player->ref);
	else
		strcpy(buf, "");
	
	return buf;
}

// this is a speech process.
void do_speak(player, saystr, type)
	object *player;
	char *saystr;
	int type;
{
	OBJECT *o;
	OBJECT *what;
	char player_name[1024];
	
	if(player->location->flags & QUIET) {
		error_report(player, M_ENOTALK);
		return;
	}
	
	for(o=player_list;o;o=o->next)
		if(o->loc == player->loc && (o->flags & CONNECT)) {
			if(type == SPEAK_SAY && o->ref != player->ref)
				notify(o, pstr("%s says, \"%s|X|\"%s", color_name(player, -1), substitute(saystr), spoof(player, o)));
			else if(type==SPEAK_SAY && o->ref == player->ref)
				notify(o, pstr("You say, \"%s|X|\"", substitute(saystr)));
			else if(type==SPEAK_POSE)
				notify(o, pstr("%s %s%s", color_name(player, -1), substitute(saystr), spoof(player, o)));
			else if(type==SPEAK_POSS)
				notify(o, pstr("%s's %s%s", color_name(player, -1), substitute(saystr), spoof(player, o)));	
			else if(type==SPEAK_THINK)
				notify(o, pstr("%s |N+|. |B|o |B+|O |C+|(|X| %s |C+|)%s",
					color_name(player, -1), substitute(saystr), spoof(player, o)));
			else if(type == SPEAK_TO) { // this gets more complicated.
				if (!(what = match_player(front(saystr)))) {
					error_report(player, M_ENOPLAY);
					return;
				}
				if(player->loc != what->loc) {
					error_report(player, M_ENOTHERE);
					return;
				}
				strcpy(player_name, color_name(player, -1));
				notify(o, pstr("%s |C+|[|Y+|to |X|%s|C+|]|W+|:|X| %s%s",
					player_name, color_name(what, -1), speech_parse(player, restof(saystr)),
						spoof(player, o)));
			}
				
		}
}

// remotely talk to a single player.
void do_page(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	OBJECT *who;	// who we're paging.
	char page_buffer[2][MAX_BUFSIZE];
	char topage[MAX_BUFSIZE], alias[MAX_BUFSIZE];

	/* Let's deconstruct any page <whoever>=<message> */
	if(strchr(arg1, '=')) {
		sprintf(page_buffer[0], "%s %s", rparse(arg1, '='), arg2);
		arg1 = fparse(arg1, '=');
	} else {
		sprintf(page_buffer[0], "%s", arg2);
	}

	if(!(who = match_player(arg1))) {	
		if( (who = match_player(atr_get(player, "LastPageTo"))) ) {
			sprintf(topage, "%s %s", arg1, page_buffer[0]);
		} else {
			error_report(player, M_ENOPLAY);
			return;
		}
	} else {
		strcpy(topage, page_buffer[0]);
	}
	
	
	if(!parse_lock(player, atr_get(who, "PlayerLock")) || !parse_lock(who, atr_get(player, "PlayerLock"))) {
		error_report(player, M_EPAGELOCK);
		return;
	}
	
	
	if(!(who->flags & CONNECT)) {
		perm_denied(player, "That player is not connected!");
		return;
	}
	
	if((who->flags & INVISIBLE) && !can_control(player, who)) {
		perm_denied(player, "That player is not connected!");
		return;
	}
	
	sprintf(alias, "|C+|(|Y+|%s|C+|)", (*strip_color((atr_get(player, "Alias")))) ? atr_get(player, "Alias") : 
		pstr("%d", player->ref));

	if(topage[0] == ':') {
		sprintf(page_buffer[0], "%s %s |C|pages|W+|: |X|%s %s", color_name(player, -1), alias,
			color_name(player, -1), topage + 1);
		sprintf(page_buffer[1], "|C|You paged |X|%s |C|with|W+|: |X|%s %s", color_name(who, -1),
			color_name(player, -1), topage + 1);
	} else if (topage[0] == ';') {
		sprintf(page_buffer[0], "%s %s |C|pages|W+|: |X|%s|C|'s|X| %s", color_name(player, -1),
			alias, color_name(player, -1), topage + 1);
		sprintf(page_buffer[1], "|C|You paged |X|%s |C|with|W+|: |X|%s|C|'s|X| %s",
			color_name(who, -1), color_name(player, -1), topage + 1);
	} else if (topage[0] == '.') {
		sprintf(page_buffer[0], "%s %s |C|pages|W+|: |X|%s |N+|. |B|o |B+|O |C+|(|X| %s |C+|)",
			color_name(player, -1), alias, color_name(player, -1), topage + 1);
		sprintf(page_buffer[1], "|C|You paged |X|%s |C|with|W+|: |X|%s |N+|. |B|o |B+|O |C+|( |X|%s |C+|)", 
			color_name(who, -1), color_name(player, -1), topage + 1);
	} else {
		sprintf(page_buffer[0], "%s %s |C|pages|W+|: |X|%s", color_name(player, -1), alias,
			topage);
		sprintf(page_buffer[1], "|C|You paged |X|%s |C|with|W+|: |X|%s", color_name(who, -1),
			topage);
	}

	sprintf(page_buffer[0], "%s%s", page_buffer[0], spoof(player, who));
	notify(who, page_buffer[0]);
	notify(player, page_buffer[1]);	
	
	SET_ATR(player, "LastPageTo", who->name);
}

// World announcements/broadcasts to all connected.
void do_wall(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	DESCRIPTOR *d;
	int official = 0;
	char cname[1024], buf[1024], format[1024];
	
	
	
	strcpy(cname, color_name(player, -1));
	if(!strcmpm(arg1, "official")) {
		// here we check if player is a wizard. only wizards can make official
		// broadcasts.
		official = 1;
		if(player->powlvl < CAN_WALLOFF) {
			error_report(player, M_EBADPOW);
			return;
		}
		if(*arg2 == ':')
			sprintf(buf, "|R+|Official |C|broadcast by|X| %s|W+|:|X| %s %s",
				cname, cname, arg2+1);
		else if (*arg2 ==';')
			sprintf(buf, "|R+|Official |C|broadcast by|X| %s|W+|:|X| %s's %s",
				cname, cname, arg2+1);
		else if(*arg2 == '.')
			sprintf(buf, "|R+|Official |C|broadcast by|X| %s|W+|:|X| %s |N+|. |B|o |B+|O |C+|(|X| %s |C+|)",
				cname, cname, arg2+1);
		else
			sprintf(buf, "|R+|Official |C|broadcast by|X| %s|W+|:|X| %s",
				cname, arg2);
	} else {
		if(player->flags & NO_WALLS) {
			error_report(player, M_EFLGSET);
			return;
		}
		if(*arg2)
			sprintf(format, " %s", arg2);
		else
			strcpy(format, "");
			
		if(*arg1 == ':')
			sprintf(buf, "%s |C+|announces|W+|:|X| %s|X| %s %s", cname, cname, arg1+1, arg2);
		else if(*arg1 == ';')
			sprintf(buf, "%s |C+|announces|W+|:|X| %s's|X| %s %s", cname, cname, arg1+1, arg2);
		else if (*arg1 == '.') 
			sprintf(buf, "%s |C+|announces|W+|:|X| %s |N+|. |B|o |B+|O |C+|(|X| %s%s |C+|)",
				cname, cname, arg1+1, format);
			
		else
			sprintf(buf, "%s |C+|announces|W+|:|X| %s %s", cname, arg1, arg2);
	}
	
	sprintf(buf, "%s\n", buf);
	for(d=descriptor_list; d; d=d->next)
		if(d->state == CONNECTED && !d->internal)
			if (official || (!(d->player->flags & NO_WALLS)) )
				dwrite(d, buf);
}
