/****************************************************************************
 MYTH - Player routines
 Coded by Saruk (01/23/99)
 ---
 Takes care of player creation and other player-specific functions.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// for crypt();
#include <unistd.h>

#include "db.h"
#include "net.h"
#include "externs.h"

// Create a player.
object *create_player(player, name, pass)
	object *player;
	char *name;
	char *pass;
{
	OBJECT *o;
	char password[1024];
	
	if(!*name || !*pass) {
		perm_denied(player, "You left out an argument.");
		return(NULL);
	}
	
	// copy the encrypted password.
	strcpy(password, mcrypt(pass));
	
	
	// check for duplicity in names.
	if(__find_player(name) || !__allowed_name(name)) {
		error_report(player, M_EILLNAME);
		return(NULL);	
	}
	
	o = create_object(player->ref, TYPE_PLAYER, name);
	SET(&o->pass, password);
	o->location = loc(o);
	o->link = o->location->ref;
	o->ownit = o;
	o->linkto = o->location;
	add_content(o->location, o);
	
	// set Quota.
	SET_ATR(player, "Quota", pstr("%d", INIT_QUOTA));
	SET_ATR(player, "ChannelQuota", pstr("%d", INIT_CHQUOTA));
	notify(player, pstr("|C+|[|Y+|CREATE|C+|]|W+|: %s |B+|has been created.", o->name)); //
	return(o);
}

void do_pcreate(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	object *o;
	
	o = create_player(player, arg1, arg2);
	
	if(!o)
		return;
	
	log_channel("log_imp", pstr("|C+|* |W+|(|C+|%s|W+|) |X|%s |C|pcreated|W+|: |X|%s",
		get_date(time(0)), color_name(player, -1), color_name(o, -1)));
					
	// add player to public channel.
	filter_command(o, "ch join public");
}
	
// Find out if a player is connected. (MOVE THIS!)
int connected_player(player)
	OBJECT *player;
{
	DESCRIPTOR *d;
	
	for(d=descriptor_list; d; d=d->next)
		if(d->state == CONNECTED)
			if(d->player == player)
				return 1;
		
	return 0;
}

// Find out if player has control over victim.
int can_control(player, victim)
	object *player;
	object *victim;
{
	if (victim->ownit == rootobj && player != rootobj)
		return 0;
				
	if(player == rootobj || player == victim->ownit || player == victim)
		return 1;
		
	if ( (player->flags & WIZARD) && (player->powlvl >= victim->ownit->powlvl) )
		return 1;

	return 0;
}

// this could get complex later on.
// Destroy all linked exits from or to a room.
void relink_and_destroy(victim)
	object *victim;
{
	object *o, *onext = NULL;
	int quota = 0;
	
	quota = atoi(atr_get(victim->ownit, "Quota"));
	for(o = exit_list; o; o=onext) {
		onext = o->next;
		if(o->linkto == victim || o->location == victim) {
			del_content(o->location, o);
			exit_list = delete_object(victim, exit_list, o->ref);
			quota++;
		}
	}
	
	SET_ATR(victim->ownit, "Quota", pstr("%d", quota));
	
	// first we check the link, if it's to the victim, we set it to 0
	// next we check the loc, if it's to the victim, we set it to the link.
	// then we move them.
	
	for(o=player_list; o; o=o->next) {
		if(o->linkto == victim) {
			o->link = 0;
			o->linkto = linkit(o);
		}
		if(o->location == victim) {
			move_to(o, o->linkto);
			o->loc = o->link;
			o->location = o->linkto;
		}
	}
	
	for(o=thing_list; o; o=o->next) {
		if(o->linkto == victim) {
			o->link = o->owner;
			o->linkto = o->ownit;
		}
		if(o->location == victim) {
			move_to(o, o->linkto);
			o->loc = o->link;
			o->location = o->linkto;
		}
	}
}

// this is the overall recycle function.
void do_recycle(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	object *victim;
	int quota = 0;
	
	// can't destroy nothing.
	if(!(victim = match_object(arg1))) {
		error_report(player, M_ENOPLAY);
		return;
	}

	// now this would totally screw us over - don't think so bub.
	if(victim == rootobj || victim->ref == 0) {	
		error_report(player, M_EWAYBAD);
		return;
	}
	
	// make sure they can do it.
	// can't do it if they don't own it, or are the victim.
	// also can't do it if the victim isn't owned by them and they aren't a wizard)
	if(!can_control(player, victim)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	if(victim->flags & LOCKED) {
		error_report(player, M_EISLOCKED);
		return;
	}
	
	quota = atoi(atr_get(victim->ownit, "Quota"));	
	// figure out what type of object they are and delete them.
	if(victim->flags & TYPE_PLAYER) {
		// tell them to use the player_nuke procedure, we don't want people accidentally
		// nuking someone. let's make it precycle. Yah, that's good.
		perm_denied(player, "Use the precycle function, instead.");
	} else if (victim->flags & TYPE_ROOM) {
		// There should be code in here to do the following:
		//	1. delete all exits linked to and from the victim.
		//	2. set all links to this room to owners (for things) and back to 0 (for players)
		//	3. notify_in(victim, exception, "destroyed this room"); (exception == NULL)
		//	4. Maybe later on do smart exit/enter checking and if the room is located between
		//	   two others, link the other two back together.
		
		// this will relink everything associated with victim and destroy all linked
		// exits.
		notify(player, pstr("|C+|[|Y+|RECYCLE|C+|]|W+|: |R|Expunging |X|%s |B+|from database.",
			color_name(victim, -1)));
		SET_ATR(victim->ownit, "Quota", pstr("%d", ++quota));
		relink_and_destroy(victim);		
		room_list = delete_object(player, room_list, victim->ref);
	} else if (victim->flags & TYPE_EXIT) {
		// this cannot be allowed. this will screw up the exit/entrance ratio. not gonna happen.
		// not gonna eat that broccoli mama. wouldn't be prudent. not at this juncture.
		// notify player that they aren't allowed to delete this victim.
		error_report(player, M_EDELEXIT);
		return;
	} else if (victim->flags & TYPE_THING) {
		// make sure we check that everyone/thing is moved back to where they're supposed to
		// be. 
		notify(player, pstr("|C+|[|Y+|RECYCLE|C+|]|W+|: |R|Expunging |X|%s |C+|(|Y+|%d|C+|) |B+|from database.",
			color_name(victim, -1), victim->ref));
		del_content(victim->location, victim);
		SET_ATR(victim->ownit, "Quota", pstr("%d", ++quota));
		relink_and_destroy(victim);
		thing_list = delete_object(player, thing_list, victim->ref);
	}		
		
}

// this searches an entire player's onwership and delete it.
void find_and_destroy(victim)
	object *victim;
{
	object *o;
	
	// let's go through the database and change the owner of any players to themselves.
	for(o=player_list; o; o=o->next)
		if(o->ownit == victim && o->ownit != victim) {
			o->owner = o->ref;
			o->ownit = o;
		}
	
	for(o=room_list; o; o=o->next)
		if(o->ownit == victim) {
			relink_and_destroy(o);
			room_list = delete_object(victim, room_list, o->ref);
		}
	
	for(o=thing_list; o; o=o->next)
		if(o->ownit == victim) {
			del_content(o->location, o);
			thing_list = delete_object(victim, thing_list, o->ref);
		}
	
}


// let's just hope the player know's what they're doing.
// because this goes through everything the player owns and wipes it all out.
// this requires a player's password.

void do_precycle(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	object *victim;
	
	if(!(victim=match_player(arg1))) {
		error_report(player, M_ENOPLAY);
		return;
	}
	
	if(!can_control(player, victim)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	if(victim->ref == 1) {
		error_report(player, M_EWAYBAD);
		return;
	}
	
	if (player->ref == victim->ref || 
		( (victim->owner != player->ref) && !(player->flags & WIZARD)) ) {
		error_report(player, M_ENOCANDO);
		return;
	}	
	
	if(dcrypt(player->pass, arg2)) {
		error_report(player, M_EBADPASS);
		return;
	}
	
	notify(victim, "|C+|[|Y+|NUKED|C+|]|W+|: |R+|Expunging |B+|you from the database.");
	if(victim->flags & GUEST) {
		boot_off(victim->ref);
		return;
	}
	
	boot_off(victim->ref);
	find_and_destroy(victim);
	log_channel("log_imp", pstr("|R+|* |W+|(|C+|%s|W+|) |X+|%s |C|precycled|W+|: |X|%s",
		get_date(time(0)), color_name(player, -1), color_name(victim, -1)));
	notify(player, pstr("|C+|[|Y+|RECYCLE|C+|]|W+|: |R|Expunging %s |B+|from the database.",
		color_name(victim, -1)));
	printf("[URGENT!]: %s was player-recycled by %s!", victim->name, player->name);
	del_content(victim->location, victim);
	player_list = delete_object(player, player_list, victim->ref);
}

void guest_precycle(player, victim)
	object *player;
	object *victim;
{
	// unless this is the root object, no one can call this.
	// also, if the victim is not a guest, not going to recycle it.
	if ( (player != rootobj) || (!(victim->flags & GUEST)) )
		return;
	
	find_and_destroy(victim);
	del_content(victim->location, victim);
	player_list = delete_object(player, player_list, victim->ref);
}

// One must be SURE which exit he is deleting, so object ids will be FORCED.
void do_remove_exit(player, arg1)
	object *player;
	char *arg1;
{
	object *check;
	
	if (!ISNUM(match_num(arg1))) {
		error_report(player, M_EOBJID);
		return;
	}
	
	if(!(check = match_exit(arg1))) {
		error_report(player, M_EBADEXIT);
		return;
	}
	
	if(!can_control(player, check)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	notify(player, pstr("|C+|[|Y+|EXIT|C+|]|W+|: |X|%s |C|expunged from database.",
		color_name(check, -1)));
	SET_ATR(check->ownit, "Quota", pstr("%d", atoi(atr_get(check->ownit, "Quota")) + 1));
	del_content(check->location, check);
	exit_list = delete_object(player, exit_list, check->ref);
}

// ANYTHING BEYOND THIS SHOULD BE MOVED TO: db/objects.c
// move player to newloc.
int move_to(player, newloc)
	object *player;
	object *newloc;
{
	CONTENTS *exception = NULL;
	
	// check newloc and return if it's an exit.
	if( (newloc->flags & TYPE_EXIT) || (player->flags & TYPE_ROOM) || (player->flags & TYPE_EXIT)) {
		error_report(player, M_EBADLNK);
		return 0;
	}
	
	add_exception(&exception, player);
	if(!(player->flags & INVISIBLE))
		notify_in(player->location, exception, pstr("%s |B+|has left.", color_name(player, -1)));
	
	del_content(player->location, player);
	player->loc = newloc->ref;
	player->location = newloc;
	add_content(player->location, player);
	if(!(player->flags & INVISIBLE))
		notify_in(player->location, exception, pstr("%s |B+|has arrived.", color_name(player, -1)));
	
	do_look(player, "");
	clear_exception(exception);
	
	return 1;
}

// join matched arg1.
void do_join(player, arg1)
	object *player;
	char *arg1;
{
	object *victim;
	
	if(!(victim = match_object(arg1))) {
		error_report(player, M_EILLNAME);
		return;
	}
	
	if(!can_control(player, victim)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	move_to(player, victim->location);
}

// summon matched arg1
void do_summon(player, arg1)
	object *player;
	char *arg1;
{
	object *victim;
	
	if(!(victim = match_object(arg1))) {
		error_report(player, M_EILLNAME);
		return;
	}
	
	if(!can_control(player, victim)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	move_to(victim, player->location);
}

void do_home(player)
	object *player;
{
	move_to(player, player->linkto);
}

void do_boot(player, arg1, arg2)
	object *player;
	char *arg1;
	char *arg2;
{
	object *victim;
	
	if(!*arg1 || !*arg2) {
		error_report(player, M_EINVARG);
		return;
	}
	
	if(!(victim = match_player(arg1))) {
		error_report(player, M_ENOPLAY);
		return;
	}
	
	if(!can_control(player, victim)) {
		error_report(player, M_ENOCNTRL);
		return;
	}
	
	printf("Player %s booted by %s for..: %s\n", victim->name, player->name, arg2);
	log_channel("log_imp", pstr("|R+|* |C|Player |X|%s |R+|booted |C|by |X|%s |C|for..|W+|: |X|%s",
		color_name(victim, -1), color_name(player, -1), arg2));
	notify(victim, pstr("|R+|* |Y+|You have been booted for...|W+|:|R+| %s", arg2));
	notify(player, pstr("|C+|[|R+|BOOT|C+|]|W+|: |C|You booted |X|%s|C|...",
		color_name(victim, -1)));
	boot_off(victim->ref);
	
}

void do_cboot(player, arg1)
	object *player;
	char *arg1;
{
	int des;
	
	IS_ARGUMENT(player, (arg1));
	
	if(!isanumber(arg1)) {
		error_report(player, M_ENOTANUM);
		return;
	}
	
	des = match_num(arg1);
	dfree(des);
	log_channel("log_imp", pstr("|R+|* |X|%s |C|cbooted descriptor|W+|: |C+|%d",
		color_name(player, -1), des));
	return;
}