/****************************************************************************
 MYTH - Trigger System
 Coded by Saruk (03/27/99)
 ---
 This is the MYTH trigger system, thinks get triggered on occasion.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "db.h"
#include "net.h"
#include "externs.h"

/* Locking exits */
int exit_lock(player, exit)
	object *player;
	object *exit;
{
	if(parse_lock(player, atr_get(exit, "Through"))) 
		return 1;
	else
		return 0;
}

void exit_trigger(player, exit)
	object *player;
	object *exit;
{
	CONTENTS *exception = NULL;
	char buf[MAX_BUFSIZE];

	if(!exit_lock(player, exit)) {
		notify(player, pstr("|Y+|*|X| %s", (*(atr_get(exit, "FThrough"))) ? atr_get(exit, "FThrough"):
			"|C|You can't go that way."));
		return;
	} else {
		if(*(atr_get(exit, "SThrough")))
			notify(player, pstr("|Y+|*|X| %s", atr_get(exit, "SThrough")));
			
	}
	
	add_exception(&exception, player);
		
	if(*(atr_get(exit, "Leave"))) {
		sprintf(buf, "%s %s", color_name(player, -1), atr_get(exit, "Leave"));
		notify_in(player->location, exception, buf);
	}

	move_to(player, exit->linkto);
	
	if(*(atr_get(exit, "Arrive"))) {
		sprintf(buf, "%s %s", color_name(player, -1), atr_get(exit, "Arrive"));
		notify_in(player->location, exception, buf);
	}
	clear_exception(exception);
}

void take_trigger(player, thing)
	object *player;
	object *thing;
{
	
	if(!parse_lock(player, atr_get(thing, "Take"))) {
		notify(player, pstr("|Y+|* |X|%s", (*(atr_get(thing, "Ftake"))) ? atr_get(thing, "FTake") : "|C|You can't take that."));
		return;
	}

	move_to(thing, player);
	notify(player, pstr("|Y+|* |X|%s", (*(atr_get(thing, "STake"))) ? atr_get(thing, "Stake") :
		"|C+|You pick it up."));
}


int pseudo_move(player, whereto, exitname)
	object *player;
	char *whereto;
	char *exitname;
{
	CONTENTS *exception = NULL;
	object *location;
	char oldloc[MAX_BUFSIZE];
	
	if(!*whereto)
		return -1;
	
	if(!(location = match_object(whereto)))
		return -1;
	
	if(!can_control(player->location->ownit, location))
		return -1;
	
	strcpy(oldloc, color_name(player->location, -1));
	add_exception(&exception, player);
	if(!(player->flags & INVISIBLE))
		notify_in(player->location, exception, pstr("|Y+|* |X|%s|C| goes through the exit marked |B+|%s",
			color_name(player, -1), exitname));
	move_to(player, location);
	if(!(player->flags & INVISIBLE))
		notify_in(player->location, exception, pstr("|Y+|* |X|%s|C| arrives from |B+|%s",
			color_name(player, -1), oldloc));
	clear_exception(exception);	
	return 1;	
}

int is_a_direction(direction)
	char *direction;
{
	if(!strcmp(direction, "south") || !strcmpm(direction, "s") || !strcmpm(direction, "north") ||
		!strcmpm(direction, "n") || !strcmpm(direction, "east") || !strcmpm(direction, "e") ||
		!strcmpm(direction, "west") || !strcmpm(direction, "w") || !strcmpm(direction, "southwest") ||
		!strcmpm(direction, "sw") || !strcmpm(direction, "southeast") || !strcmpm(direction, "se") ||
		!strcmpm(direction, "northwest") || !strcmpm(direction, "nw") || !strcmpm(direction, "northeast") ||
		!strcmpm(direction, "ne") || !strcmpm(direction, "up") || !strcmpm(direction, "u") || 
		!strcmpm(direction, "down") || !strcmpm(direction, "d"))
			return -1;

	return 0;
}

int match_pseudo_exit(player, matchit)
	object *player;
	char *matchit;
{
	attr *a;
	
	for(a = player->location->alist; a; a=a->next)
		if(a->flags & AEXIT) {
			if(!strcmpm(matchit, "south") || !strcmpm(matchit, "s"))
				return pseudo_move(player, atr_get(player->location, "South"), "South");
			else if (!strcmpm(matchit, "north") || !strcmpm(matchit, "n"))
				return pseudo_move(player, atr_get(player->location, "North"), "North");
			else if (!strcmpm(matchit, "east") || !strcmpm(matchit, "e"))
				return pseudo_move(player, atr_get(player->location, "East"), "East");
			else if (!strcmpm(matchit, "west") || !strcmpm(matchit, "w"))
				return pseudo_move(player, atr_get(player->location, "West"), "West");
			else if (!strcmpm(matchit, "southwest") || !strcmpm(matchit, "sw"))
				return pseudo_move(player, atr_get(player->location, "SouthWest"), "SouthWest");
			else if (!strcmpm(matchit, "southeast") || !strcmpm(matchit, "se"))
				return pseudo_move(player, atr_get(player->location, "SouthEast"), "SouthEast");
			else if (!strcmpm(matchit, "northwest") || !strcmpm(matchit, "nw"))
				return pseudo_move(player, atr_get(player->location, "NorthWest"), "NorthWest");
			else if (!strcmpm(matchit, "northeast") || !strcmpm(matchit, "ne"))
				return pseudo_move(player, atr_get(player->location, "NorthEast"), "NorthEast");
			else if (!strcmpm(matchit, "up") || !strcmpm(matchit, "u"))
				return pseudo_move(player, atr_get(player->location, "Up"), "Up");
			else if (!strcmpm(matchit, "down") || !strcmpm(matchit, "d"))
				return pseudo_move(player, atr_get(player->location, "Down"), "Down");
			else
				return 0;
		}

	if (is_a_direction(matchit))
		return -1;
		
	return 0;
}

char *show_pseudo_exit(exitname)
	char *exitname;
{
	if(!strcmpm(exitname, "south"))
		return("|C|South |W+|<|G+|S|W+|>");
	else if (!strcmpm(exitname, "north"))
		return("|C|North |W+|<|G+|N|W+|>");
	else if (!strcmpm(exitname, "east"))
		return("|C|East |W+|<|G+|E|W+|>");
	else if (!strcmpm(exitname, "west"))
		return("|C|West |W+|<|G+|W|W+|>");
	else if (!strcmpm(exitname, "southwest"))
		return("|C|SouthWest |W+|<|G+|SW|W+|>");
	else if (!strcmpm(exitname, "southeast")) 
                return("|C|SouthEast |W+|<|G+|SE|W+|>");
	else if (!strcmpm(exitname, "northwest")) 
                return("|C|NorthWest |W+|<|G+|NW|W+|>");
        else if (!strcmpm(exitname, "northeast"))
        	return("|C|NorthEast |W+|<|G+|NE|W+|>");
        else if (!strcmpm(exitname, "up"))
        	return("|C|Up |W+|<|G+|U|W+|>");
        else if (!strcmpm(exitname, "down"))
        	return("|C|Down |W+|<|G+|D|W+|>");
	else
		return "";
}

int match_real_exit(player, exitkey)
	object *player;
	char *exitkey;
{
	CONTENTS *c;
	
	for(c = player->location->contents; c; c=c->next)
		if(c->content->flags & TYPE_EXIT)
			if(match_key(c->content->pass, exitkey)) {
				exit_trigger(player, c->content);
				return TRUE;
			}
	
	return FALSE;
}